/*
 * @description: 
 * @Author: CP
 * @Date: 2020-08-18 12:28:59
 * @FilePath: \construction_management_backstage\app\base\base_controller.js
 */
'use strict';

/**
 * 控制器基类
 *
 * @author CaiAoLin
 * @date 2017/10/11
 * @version
 */
const moment = require('moment');
const messageType = require('../const/message_type');
const Controller = require('egg').Controller;
class BaseController extends Controller {

    /**
     * 构造函数
     *
     * @param {Object} ctx - egg全局context
     * @return {void}
     */
    constructor(ctx) {
        super(ctx);
        this.messageType = messageType;
        this.jsValidator = this.app.jsValidator;
    }

    /**
     * 渲染layout
     *
     * @param {String} view - 渲染的view
     * @param {Object} data - 渲染的数据
     * @param {String} modal - 渲染的modal
     * @return {void}
     */
    async layout(view, data = {}, modal = '') {
        
        data.moment = moment;
        // 获取消息提示
        const message = this.ctx.session.message;
        // 取出后删除
        this.ctx.session.message = null;

        // 获取菜单栏数据
        const menuData = await this.ctx.service.permission.getMenu();
        const sortData = await this.ctx.service.permission.sortMenuByPermission(menuData);

        const viewString = await this.ctx.renderView(view, data);
        const modalString = modal === '' ? '' : await this.ctx.renderView(modal, data);
        const renderData = {
            content: viewString,
            message: JSON.stringify(message),
            modal: modalString,
            menuData: sortData,
            dropDownMenu: data.dropDownMenu === undefined ? [] : data.dropDownMenu,
            breadCrumb: data.breadCrumb === undefined ? '' : data.breadCrumb,
            cbClass: data.cbClass,
        };
  
        await this.ctx.render('layout/layout.ejs', renderData);
    }

    /**
     * 设置提示
     *
     * @param {String} message - 提示信息
     * @param {String} type - 提示类型
     * @return {void}
     */
    setMessage(message, type) {
        let icon = '';
        switch (type) {
            case messageType.SUCCESS:
                icon = 'check';
                break;
            case messageType.ERROR:
                icon = 'exclamation-circle';
                break;
            case messageType.INFO:
                icon = 'info-circle';
                break;
            case messageType.WARNING:
                icon = 'warning';
                break;
            default:
                break;
        }
        this.ctx.session.message = {
            type,
            icon,
            message,
        };
    }
}

module.exports = BaseController;
