'use strict';

/**
 * 客户管理相关控制器
 *
 * @author CaiAoLin
 * @date 2017/11/14
 * @version
 */
const provinceConst = require('../const/province_const');
const companyConst = require('../const/company_const');
module.exports = app => {

    class CustomerController extends app.BaseController {

        /**
         * 演示版客户页面
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async demo(ctx) {
            // 过滤客户数据
            ctx.service.customer.searchFilter(ctx.request.query);
            const customerList = await ctx.service.customer.getListWithBuilder();
            // 获取总数据量
            const total = await ctx.service.customer.count();
            // 分页相关
            const pageInfo = {
                page: ctx.page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            const renderData = {
                customerList,
                pageInfo,
                provinceList: provinceConst.provinceList,
                provinceList2: JSON.stringify(provinceConst.provinceList),
                companyType: companyConst.companyType,
                companyType2: JSON.stringify(companyConst.companyType),
                companyScale: companyConst.companyScale,
            };
            await this.layout('customer/demo.ejs', renderData);
        }

        /**
         * 检测是否存在对应通行账号数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async customerExist(ctx) {
            let result = false;
            const customerAccount = ctx.request.body.account;

            try {
                if (customerAccount === undefined || customerAccount === '') {
                    throw '数据为空';
                }

                const customerData = await ctx.service.customer.getDataInfo(customerAccount);
                result = customerData !== null;
            } catch (error) {
                result = false;
            }

            ctx.body = result;
        }
    }

    return CustomerController;
};
