'use strict';

/**
 * 合同支付 控制器
 *
 * @author Mai
 * @date 2020/3/4
 * @version
 */

const payType = {
    normal: 1,
    yf: 2,
    sf: 3,
    wc: 4
};

const payTemplate = [
    { order: 1, name: '本期应付', ptype: payType.yf, minus: false, expr: null, sexpr: null, rexpr: null },
    { order: 2, name: '本期实付', ptype: payType.sf, minus: false },
    { order: 3, name: '本期完成计量', ptype: payType.wc, minus: false, expr: 'bqwc' },
    { order: 4, name: '质量保证金', ptype: payType.normal, minus: true },
    { order: 5, name: '扣回开工预付款', ptype: payType.normal, minus: true, expr: '(bqwc/htj)*2*kgyfk', sexpr: 'htj*30%', rexpr: 'kgyfk' },
];

const calcBase = [
    { name: '签约合同价', code: 'htj', sort: 10 },
    { name: '暂列金额', code: 'zlje', sort: 2 },
    { name: '签约合同价(不含暂列金)', code: 'htjszl', sort: 1 },
    { name: '签约开工预付款', code: 'kgyfk', sort: 2 },
    { name: '签约材料预付款', code: 'clyfk', sort: 2 },
    { name: '本期完成计量', code: 'bqwc', limit: true, sort: 10 },
    { name: '100章本期完成计量', code: 'ybbqwc', limit: true, sort: 1 },
];

module.exports = app => {

    class DealPayController extends app.BaseController {

        /**
         * 项目列表
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const projectData = ctx.session.sessionProjectData;
            try {
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                const curProjectData = await ctx.service.project.getDataById(projectData.id);
                let dealpay = JSON.parse(JSON.stringify(payTemplate));
                try {
                    if (curProjectData.dealpay_json) {
                        dealpay = JSON.parse(curProjectData.dealpay_json);
                    }
                } catch (err) {
                    this.setMessage('合同支付数据错误，将重新初始化数据。如需查询详细错误，请勿修改或保存合同支付数据。', this.messageType.ERROR);
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                // 渲染数据
                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    breadCrumb,
                    payType,
                    dealpay: dealpay,
                    calcBase,
                };
                await this.layout('project/deal_pay.ejs', renderData);
            } catch (error) {
                ctx.helper.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        async save(ctx) {
            const projectData = ctx.session.sessionProjectData;
            try {
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                const data = ctx.request.body.data;
                await ctx.service.project.update({ dealpay_json: data }, { id: projectData.id });
                ctx.redirect('/project/dealpay');
            } catch (error) {
                ctx.helper.log(error);
                if (error.stack) {
                    this.setMessage('保存合同支付数据失败', this.messageType.ERROR);
                } else {
                    this.setMessage(error.toString(), this.messageType.ERROR);
                }
            }
        }

        async reset(ctx) {
            const projectData = ctx.session.sessionProjectData;
            try {
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                await ctx.service.project.update({ dealpay_json: JSON.stringify(payTemplate) }, { id: projectData.id });
                ctx.redirect('/project/dealpay');
            } catch (error) {
                ctx.helper.log(error);
                if (error.stack) {
                    this.setMessage('重置合同支付数据失败', this.messageType.ERROR);
                } else {
                    this.setMessage(error.toString(), this.messageType.ERROR);
                }
            }
        }

        /**
         * 获取共用面包屑
         *
         * @param {Object} projectData - 项目数据
         * @return {String} - 返回面包屑
         */
        _getBreadCrumb(projectData) {
            // 面包屑
            const name = projectData.project_id !== 0 ? '项目版客户' : '企业版客户';
            const breadCrumb = [
                { name: this.ctx.topPermission.name },
                { name: name, url: this.ctx.controllerName },
                { name: projectData.name },
            ];
            return this.ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
        }
    }

    return DealPayController;
};