'use strict';

/**
 * 用户组相关
 *
 * @author CaiAoLin
 * @date 2017/9/30
 * @version
 */

module.exports = app => {

    class GroupController extends app.BaseController {

        /**
         * 用户组列表
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async index(ctx) {
            // 自动获取page并分页
            const groupList = await ctx.service.group.getGroupList();
            const total = await ctx.service.group.count();

            // 分页相关
            const pageInfo = {
                page: ctx.page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            const dropDownMenu = [
                { name: '新增用户组', url: ctx.controllerName + '/add' },
            ];
            const renderData = {
                groupList,
                pageInfo,
                dropDownMenu,
            };
            await this.layout('group/index.ejs', renderData);
        }

        /**
         * 新增用户组页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async add(ctx) {
            // 获取验证规则
            const rule = ctx.service.group.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();

            // 获取菜单数据
            const menuData = await ctx.service.permission.getMenu();

            let breadCrumb = [
                { name: '用户组列表', url: ctx.controllerName },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
            const renderData = {
                jsValidator,
                menuData,
                groupData: {},
                breadCrumb,
            };
            await this.layout('group/save.ejs', renderData);
        }

        /**
         * 修改用户组页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void|Object} - 不返回或者直接跳转到上一页面
         */
        async modify(ctx) {
            try {
                const id = ctx.params.id;
                // 获取验证规则
                const rule = ctx.service.group.rule();
                const jsValidator = await this.jsValidator.convert(rule).build();
                // 获取菜单数据
                const menuData = await ctx.service.permission.getMenu();
                // 获取用户组数据
                const groupData = await ctx.service.group.getDataById(id);

                if (groupData === null) {
                    throw '不存在对应数据';
                }

                groupData.permission = groupData.permission.split(',');

                let breadCrumb = [
                    { name: '用户组列表', url: ctx.controllerName },
                ];
                breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
                const renderData = {
                    jsValidator,
                    menuData,
                    groupData,
                    breadCrumb,
                };
                await this.layout('group/save.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }

        }

        /**
         * 用户组保存
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 跳转页面
         */
        async save(ctx) {
            try {
                let id = ctx.params.id;
                id = parseInt(id);

                // 获取校验规则
                const rule = ctx.service.group.rule();
                ctx.helper.validate(rule);

                const result = await ctx.service.group.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                this.setMessage('保存成功', this.messageType.SUCCESS);
                ctx.redirect('/group');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }
        }

    }

    return GroupController;
};
