'use strict';

/**
 *
 *
 * @author Mai
 * @date
 * @version
 */

const path = require('path');
const fs = require('fs');
const analysisDefine = require('../lib/rpt_data_analysis').analysisDefine;
const messageType = require('../const/message_type');
const ctx_helper_define = [
    {key: 'add', note: "    /**\n" +
        "     * 加法 num1 + num2\n" +
        "     * @param num1\n" +
        "     * @param num2\n" +
        "     * @returns {number}\n" +
        "     */\n" +
        "    add(num1, num2)"},
    {key: 'sub', note: "    /**\n" +
        "     * 减法 num1 - num2\n" +
        "     * @param num1\n" +
        "     * @param num2\n" +
        "     * @returns {number}\n" +
        "     */\n" +
        "    sub(num1, num2)"},
    {key: 'mul', note: "    /**\n" +
        "     * 乘法 num1 * num2\n" +
        "     * @param num1\n" +
        "     * @param num2\n" +
        "     * @returns {*}\n" +
        "     */\n" +
        "    mul(num1, num2, digit = 6)"},
    {key: 'div', note: "    /**\n" +
        "     * 除法 num1 / num2\n" +
        "     * @param num1 - 被除数\n" +
        "     * @param num2 - 除数\n" +
        "     * @returns {*}\n" +
        "     */\n" +
        "    div(num1, num2, digit = 6)"},
    {key: 'round', note: "    /**\n" +
        "     * 四舍五入(统一，方便以后万一需要置换)\n" +
        "     * @param {Number} value - 舍入的数字\n" +
        "     * @param {Number} decimal - 要保留的小数位数\n" +
        "     * @returns {*}\n" +
        "     */\n" +
        "    round(value, decimal)"},
    {key: 'checkZero', note: "    /**\n" +
        "     * 检查数字是否为0\n" +
        "     * @param {Number} value\n" +
        "     * @return {boolean}\n" +
        "     */\n" +
        "    checkZero(value)"},
    {key: 'checkNumberEqual', note: "    /**\n" +
        "     * 检查数字是否相等\n" +
        "     * @param {Number} value1\n" +
        "     * @param {Number} value2\n" +
        "     * @returns {boolean}\n" +
        "     */\n" +
        "    checkNumberEqual(value1, value2)"},
    {key: 'compareCode', note: "    /**\n" +
        "     * 比较编码\n" +
        "     * @param str1\n" +
        "     * @param str2\n" +
        "     * @param symbol\n" +
        "     * @returns {number} " +
        "     */\n" +
        "    compareCode(str1, str2, symbol = '-')"},
    {key: 'getChapterCode', note: "    /**\n" +
        "     * 根据 清单编号 获取 章级编号\n" +
        "     * @param code\n" +
        "     * @param symbol\n" +
        "     * @returns {string}\n" +
        "     */\n" +
        "    getChapterCode(code, symbol = '-')"},
    {key: 'addDebugInfo', note: "    /**\n" +
        "     * 添加debug信息\n" +
        "     * 在debug模式下，debug信息将传输到浏览器并打印\n" +
        "     *\n" +
        "     * @param {String}key\n" +
        "     * @param {*}data 允许输入不定个数的参数\n" +
        "     */\n" +
        "    addDebugInfo(key, ...data)\n" +
        "    \n" +
        "    //示例代码：\n" +
        "    let bCode=$JE.F(2510,$CURRENT_RPT);\n" +
        "    let mLen=$JE.getFieldDataLen(bCode, $CURRENT_DATA);\n" +
        "    \n" +
        "    $CTX_HELPER.addDebugInfo('mLen', mLen);\n" +
        "    for (let i=0; i&lt;mLen; i++) {\n" +
        "        let mno1=$JE.getFieldValue(bCode, $CURRENT_DATA, i, '');\n" +
        "        $CTX_HELPER.addDebugInfo('', '清单编号',mno1);\n" +
        "       // console.log('清单编号',mno1);\n" +
        "    }", img: '/public/images/help/rpt-index-addDebugInfo.png', imgInfo: '示例结果如下：'},
    //{key: 'add', note: ""},
];

module.exports = app => {

    class helperController extends app.BaseController {

        async index (ctx) {
            const renderData = {
                cbClass: 'help-content',
                analysisDefine,
                ctx_helper_define
            };
            try {
                if (ctx.query.m && ctx.query.m !== '') {
                    let htmlPath = 'help';
                    htmlPath = path.join(htmlPath, ctx.query.m);
                    if (ctx.query.s && ctx.query.s !== '') {
                        htmlPath = path.join(htmlPath, ctx.query.s);
                        if (ctx.query.d && ctx.query.d !== '') {
                            htmlPath = path.join(htmlPath, ctx.query.d);
                        }
                    }
                    if (fs.existsSync(path.join(ctx.app.baseDir, 'app/view',  htmlPath + '.ejs'))) {
                        await this.layout(htmlPath + '.ejs', renderData);
                    } else {
                        throw '您查询的帮助暂不存在，请耐心等待小编完善';
                    }
                } else {
                    await this.layout('help/index.ejs', renderData);
                }
            } catch(err) {
                ctx.helper.log(err);
                if (err.stack) {
                    this.setMessage('查询帮助信息错误', messageType.ERROR);
                } else {
                    this.setMessage(err, messageType.INFO);
                }
                ctx.redirect('/help');
            }
        }
    }

    return helperController;
};