/*
 * @description: 
 * @Author: CP
 * @Date: 2020-08-18 12:29:00
 * @FilePath: \construction_management_backstage\app\controller\login_controller.js
 */
'use strict';

/**
 * 登录相关控制器
 *
 * @author CaiAoLin
 * @date 2017/8/29
 * @version
 */

// 加密
const crypto = require('crypto');
// 验证码
const Captcha = require('../lib/captcha');
module.exports = app => {

    class LoginController extends app.Controller {

        /**
         * 登录页面
         *
         * @param {object} ctx - egg全局context
         * @return {void}
         */
        async index(ctx) {
            // 获取消息提示
            const loginError = this.ctx.session.loginError;
            // 取出后删除
            this.ctx.session.loginError = null;

            const renderData = {
                loginError,
            };
            await ctx.render('login/login.ejs', renderData);
        }

        /**
         * 验证码注册
         *
         * @param {object} ctx - egg全局context
         * @return {void}
         */
        async captcha(ctx) {
            const captcha = new Captcha(app);
            const response = await captcha.register(ctx);
            ctx.body = response;
        }

        /**
         * 登录操作
         *
         * @param {object} ctx - egg全局context
         * @return {void}
         */
        async login(ctx) {
            const username = ctx.request.body.username;
            const password = ctx.request.body.password;

            try {
                const captcha = new Captcha(app);
                const captchaResult = await captcha.validate(ctx);
                if (!captchaResult) {
                    throw '验证码错误';
                }
                // 获取校验规则
                const rule = ctx.service.manager.rule('login');
                ctx.validate(rule);
              
                // 如果不是admin则进入cld模式判断
                const result = await ctx.service.manager.validManager(username, password);
                if (!result) {
                    throw '用户名或密码错误';
                }
            } catch (error) {
                console.log(error);
                ctx.session.loginError = error;
            }

            ctx.redirect('/dashboard');
        }

        /**
         * 退出操作
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async logout(ctx) {
            // 删除session并跳转
            ctx.session = null;
            ctx.redirect('/');
        }

    }

    return LoginController;
};
