'use strict';

/**
 * 消息管理控制器
 *
 * @author CaiAoLin
 * @date 2017/10/25
 * @version
 */

module.exports = app => {
    class MessageController extends app.BaseController {

        /**
         * 消息列表
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;

            // 筛选数据
            ctx.sort = ['id', 'DESC'];
            ctx.service.message.searchFilter(ctx.request.query);
            // 获取数据
            const messageList = await ctx.service.message.getListWithBuilder();
            const total = await ctx.service.message.count({ type: 2 });

            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            const dropDownMenu = [
                { name: '添加新消息', url: ctx.controllerName + '/add' },
            ];

            const renderData = {
                messageList: JSON.parse(JSON.stringify(messageList).replace(/\\r\\n/g, '<br>').replace(/\\"/g, '&#34;').replace(/'/g, '&#39;').replace(/\\t/g, '&#9;')),
                pageInfo,
                status: ctx.service.message.displayStatus,
                dropDownMenu,
            };
            await this.layout('message/index.ejs', renderData, 'message/modal.ejs');
        }

        /**
         * 新增消息页面
         *
         * @param {Object} ctx - egg 全局变量
         * @return {void}
         */
        async add(ctx) {
            // 获取规则
            const rule = ctx.service.message.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();

            let breadCrumb = [
                { name: '消息通知', url: ctx.controllerName },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
            const renderData = {
                jsValidator,
                messageData: {},
                breadCrumb,
            };
            await this.layout('message/save.ejs', renderData);
        }

        /**
         * 编辑消息页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async modify(ctx) {
            try {
                let id = ctx.params.id;
                id = parseInt(id);

                const rule = ctx.service.message.rule();
                const jsValidator = await this.jsValidator.convert(rule).build();
                const messageData = await ctx.service.message.getDataById(id);
                if (messageData === null) {
                    throw '不存在对应管理员';
                }

                let breadCrumb = [
                    { name: '消息通知', url: ctx.controllerName },
                ];
                breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);

                const renderData = {
                    jsValidator,
                    messageData,
                    breadCrumb,
                };
                await this.layout('message/save.ejs', renderData);

            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 消息保存操作
         *
         * @param {Object} ctx - egg 全局变量
         * @return {void}
         */
        async save(ctx) {
            try {
                const rule = ctx.service.message.rule();
                ctx.validate(rule);

                const id = ctx.params.id;
                const result = await ctx.service.message.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                this.setMessage('保存消息数据成功', this.messageType.SUCCESS);
                ctx.redirect('/message');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除消息
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);
            try {
                if (isNaN(id) || id <= 0) {
                    throw 'id错误';
                }
                const result = await ctx.service.message.deleteById(id);
                if (!result) {
                    throw '删除失败';
                }

                this.setMessage('删除成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }
    }

    return MessageController;
};
