'use strict';

/**
 * 项目账户相关控制器
 *
 * @author CaiAoLin
 * @date 2018/1/25
 * @version
 */

const accountGroupConst = require('../const/account_group');
module.exports = app => {

    class ProjectAccountController extends app.BaseController {

        /**
         * 项目账号页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const projectData = ctx.session.sessionProjectData;
            try {
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                const page = ctx.page;

                // 获取用户数据
                ctx.service.projectAccount.searchFilter(ctx.request.query, projectData.id);
                const total = await ctx.service.projectAccount.count({ project_id: projectData.id });
                // const accountList = await ctx.service.projectAccount.getAccountByProjectId(projectData.id);
                const accountList = await ctx.service.projectAccount.getListWithBuilder();

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);

                // 分页相关
                const pageInfo = {
                    page,
                    total: Math.ceil(total / app.config.pageSize),
                    queryData: JSON.stringify(ctx.urlInfo.query),
                };
                // 渲染数据
                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    accountList,
                    breadCrumb,
                    accountGroup: accountGroupConst.group,
                    pageInfo,
                };
                await this.layout('project/account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 添加账号页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async add(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 获取验证规则
                const rule = ctx.service.projectAccount.rule('add');
                const frontRule = ctx.helper.validateConvert(rule);

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    accountData: {},
                    rule: JSON.stringify(frontRule),
                    breadCrumb,
                    accountGroup: accountGroupConst.group,
                    accountGroup2: JSON.stringify(accountGroupConst.group),
                };
                await this.layout('project/save_account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 修改账号页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async modify(ctx) {
            const projectData = ctx.session.sessionProjectData;
            let accountId = ctx.params.id;
            try {
                accountId = parseInt(accountId);
                if (isNaN(accountId) || accountId <= 0) {
                    throw '参数错误';
                }
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 获取验证规则
                const rule = ctx.service.projectAccount.rule('modify');
                const frontRule = ctx.helper.validateConvert(rule);

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                const accountData = await ctx.service.projectAccount.getDataByCondition({ id: accountId });
                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    accountData,
                    rule: JSON.stringify(frontRule),
                    breadCrumb,
                    accountGroup: accountGroupConst.group,
                    accountGroup2: JSON.stringify(accountGroupConst.group),
                };
                await this.layout('project/save_account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 修改登录账号名页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async modifyAccount(ctx) {
            const projectData = ctx.session.sessionProjectData;
            let accountId = ctx.params.id;
            try {
                accountId = parseInt(accountId);
                if (isNaN(accountId) || accountId <= 0) {
                    throw '参数错误';
                }
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                const accountData = await ctx.service.projectAccount.getDataByCondition({ id: accountId });
                const renderData = {
                    projectData,
                    accountData,
                    breadCrumb,
                };
                await this.layout('project/modify_account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 修改登录密码页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async modifyPassword(ctx) {
            const projectData = ctx.session.sessionProjectData;
            let accountId = ctx.params.id;
            try {
                accountId = parseInt(accountId);
                if (isNaN(accountId) || accountId <= 0) {
                    throw '参数错误';
                }
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                const accountData = await ctx.service.projectAccount.getDataByCondition({ id: accountId });
                const renderData = {
                    projectData,
                    accountData,
                    breadCrumb,
                };
                await this.layout('project/modify_password.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 修改副密码页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async modifyBackdoor(ctx) {
            const projectData = ctx.session.sessionProjectData;
            let accountId = ctx.params.id;
            try {
                accountId = parseInt(accountId);
                if (isNaN(accountId) || accountId <= 0) {
                    throw '参数错误';
                }
                // 验证数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                const accountData = await ctx.service.projectAccount.getDataByCondition({ id: accountId });
                const renderData = {
                    projectData,
                    accountData,
                    breadCrumb,
                };
                await this.layout('project/modify_backdoor.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除账户
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async delete(ctx) {
            const projectData = ctx.session.sessionProjectData;
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                if (projectData.id === undefined) {
                    throw '不存在对应项目';
                }

                const result = await ctx.service.projectAccount.deleteById(id);
                if (!result) {
                    throw '删除账户失败';
                }

                this.setMessage('删除账户成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 检测账户是否存在
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async accountExist(ctx) {
            const projectData = ctx.session.sessionProjectData;
            const account = ctx.request.body.account;
            const extraId = ctx.request.body.extraId !== undefined ? ctx.request.body.extraId : '';
            let result = false;

            try {
                if (projectData.id === undefined) {
                    throw '不存在对应项目';
                }
                result = await ctx.service.projectAccount.isAccountExist(account, projectData.id, extraId);
            } catch (error) {
                result = false;
            }

            ctx.body = !result;
        }

        /**
         * 账号数据保存操作
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async save(ctx) {
            const projectData = ctx.session.sessionProjectData;
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                const scene = id > 0 ? 'modify' : 'add';
                // 验证数据
                const rule = ctx.service.projectAccount.rule(scene);
                ctx.validate(rule);

                const result = await ctx.service.projectAccount.save(ctx.request.body, id);
                if (!result) {
                    throw '保存账号数据失败';
                }

                this.setMessage('保存账号数据成功', this.messageType.SUCCESS);
                ctx.redirect('/' + ctx.controllerName + '/account');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 项目设置 -- 账号启用和停用设置(Post)
         * @param ctx
         * @return {Promise<void>}
         */
        async accountSwitch(ctx) {
            const responseData = {
                err: 0, msg: '', data: null,
            };
            try {
                // 获取项目数据
                const projectId = ctx.session.sessionProjectData.id;
                // const projectData = await ctx.service.project.getDataById(projectId);
                // if (projectData === null) {
                //     throw '没有对应的项目数据';
                // }
                // if (ctx.session.sessionUser.is_admin === 0) {
                //     throw '没有访问权限';
                // }
                const data = JSON.parse(ctx.request.body.data);
                const result = await ctx.service.projectAccount.update(data, { id: data.id });
                if (!result) {
                    throw '提交数据失败';
                }
            } catch (err) {
                this.log(err);
                responseData.err = 1;
                responseData.msg = err;
            }
            ctx.body = responseData;
        }

        /**
         * 项目设置 -- 账号解绑设置(Post)
         * @param ctx
         * @return {Promise<void>}
         */
        async accountUnbind(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                const accountId = parseInt(ctx.request.body.id);
                const result = await ctx.service.projectAccount.update({ bind: 0 }, { id: accountId });
                if (!result) {
                    throw '解绑失败';
                }

                this.setMessage('解绑成功', this.messageType.SUCCESS);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
            ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 重置用户密码
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async resetPassword(ctx) {
            let accountId = ctx.params.id;
            const response = {
                err: 0,
                msg: '',
            };
            try {
                accountId = parseInt(accountId);
                let password = ctx.request.body.reset_password;
                const account = ctx.request.body.account !== undefined ? ctx.request.body.account : '';
                const backdoor = ctx.request.body.backdoor !== undefined ? ctx.request.body.backdoor : false;
                password = password.toString();
                if (isNaN(accountId) || accountId <= 0 || password.length < 6) {
                    throw '参数错误';
                }
                let result = true;
                if (backdoor) {
                    result = await ctx.service.projectAccount.resetBackdoorPassword(accountId, password);
                } else {
                    result = await ctx.service.projectAccount.resetPassword(accountId, password, account);
                }

                if (!result) {
                    throw '重置密码失败!';
                }
            } catch (error) {
                response.err = 1;
                response.msg = error;
            }

            ctx.body = response;
        }

        /**
         * 获取共用面包屑
         *
         * @param {Object} projectData - 项目数据
         * @return {String} - 返回面包屑
         */
        _getBreadCrumb(projectData) {
            // 面包屑
            const name = projectData.project_id !== 0 ? '项目版客户' : '企业版客户';
            const breadCrumb = [
                { name: this.ctx.topPermission.name },
                { name: name, url: this.ctx.controllerName },
                { name: projectData.name },
            ];
            return this.ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
        }


        /**
         * 项目账号页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async enterpriseIndex(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;
            try {
                // 验证数据
                if (enterpriseData.id === undefined) {
                    throw '不存在对应的企业数据';
                }

                // 获取用户数据
                ctx.service.projectAccount.searchFilter(ctx.request.query);
                const accountList = await ctx.service.projectAccount.getAccountByEnterpriseId(enterpriseData.id);

                // 面包屑
                const breadCrumb = this._getBreadCrumb(enterpriseData);
                // 渲染数据
                const renderData = {
                    enterpriseData,
                    status: ctx.service.enterprise.displayStatus,
                    statusClass: ctx.service.enterprise.statusClass,
                    accountList,
                    breadCrumb,
                };
                await this.layout('enterprise/account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 添加账号页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async enterpriseAdd(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;

            try {
                // 验证数据
                if (enterpriseData.id === undefined) {
                    throw '不存在对应的企业数据';
                }

                // 获取验证规则
                const rule = ctx.service.projectAccount.rule('add');
                const frontRule = ctx.helper.validateConvert(rule);

                // 面包屑
                const breadCrumb = this._getBreadCrumb(enterpriseData);
                const renderData = {
                    enterpriseData,
                    status: ctx.service.enterprise.displayStatus,
                    statusClass: ctx.service.enterprise.statusClass,
                    accountData: {},
                    rule: JSON.stringify(frontRule),
                    breadCrumb,
                };
                await this.layout('enterprise/save_account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 修改账号页面
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async enterpriseModify(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;
            let accountId = ctx.params.id;
            try {
                accountId = parseInt(accountId);
                if (isNaN(accountId) || accountId <= 0) {
                    throw '参数错误';
                }
                // 验证数据
                if (enterpriseData.id === undefined) {
                    throw '不存在对应的企业数据';
                }

                // 获取验证规则
                const rule = ctx.service.projectAccount.rule('modify');
                const frontRule = ctx.helper.validateConvert(rule);

                // 面包屑
                const breadCrumb = this._getBreadCrumb(enterpriseData);
                const accountData = await ctx.service.projectAccount.getDataByCondition({ id: accountId });
                const renderData = {
                    enterpriseData,
                    status: ctx.service.enterprise.displayStatus,
                    statusClass: ctx.service.enterprise.statusClass,
                    accountData,
                    rule: JSON.stringify(frontRule),
                    breadCrumb,
                };
                await this.layout('enterprise/save_account.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除账户
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async enterpriseDelete(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                if (enterpriseData.id === undefined) {
                    throw '不存在对应企业';
                }

                const result = await ctx.service.projectAccount.deleteById(id);
                if (!result) {
                    throw '删除账户失败';
                }

                this.setMessage('删除账户成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 检测账户是否存在
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async enterpriseAccountExist(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;
            const account = ctx.request.body.account;
            let result = true;

            try {
                if (enterpriseData.id === undefined) {
                    throw '不存在对应项目';
                }
                result = await ctx.service.projectAccount.isEnterpriseAccountExist(account, enterpriseData.id);
            } catch (error) {
                result = true;
            }

            ctx.body = !result;
        }

        /**
         * 账号数据保存操作
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async enterpriseSave(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (enterpriseData.id === undefined) {
                    throw '不存在对应的企业数据';
                }
                const scene = id > 0 ? 'modify' : 'add';
                // 验证数据
                const rule = ctx.service.projectAccount.rule(scene);
                ctx.validate(rule);

                const result = await ctx.service.projectAccount.enterpriseSave(ctx.request.body, id);
                if (!result) {
                    throw '保存账号数据失败';
                }

                this.setMessage('保存账号数据成功', this.messageType.SUCCESS);
                ctx.redirect('/' + ctx.controllerName + '/account');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }
    }

    return ProjectAccountController;
};
