'use strict';

/**
 * 项目管理控制器
 *
 * @author CaiAoLin
 * @date 2017/11/8
 * @version
 */

const cldOfficeConst = require('../const/cld_office');
const pageShowConst = require('../const/page_show');
const path = require('path');
const sendToWormhole = require('stream-wormhole');
module.exports = app => {

    class ProjectController extends app.BaseController {

        /**
         * 项目列表
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;
            // 过滤数据
            ctx.service.project.searchFilter(ctx.request.query, ctx.session.managerSession);
            // 获取项目列表
            const total = await ctx.service.project.getCountWithBuilder();
            const projectList = await ctx.service.project.getListWithBuilder();
            // const filter = ctx.session.managerSession.office !== cldOfficeConst.type.ZONGBU ? { office: ctx.session.managerSession.office !== cldOfficeConst.type.JIANGXI ? ctx.session.managerSession.office : [cldOfficeConst.type.JIANGXI, cldOfficeConst.type.HUNAN] } : '';
            // const total = await ctx.service.project.count(filter);
            if (projectList !== null) {
                // 整理数据
                let i = 0;
                for (const project of projectList) {
                    const manager = await ctx.service.manager.getDataById(project.manager_id);
                    projectList[i].manager_username = manager !== null ? manager.username : '';
                    i++;
                }
            }

            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            // 下拉菜单
            const dropDownMenu = [
                { name: '新增项目', url: ctx.controllerName + '/add' },
            ];

            const managerSession = ctx.session.managerSession;
            let officeList2 = [];
            const permission = managerSession.permission;
            if (permission === 'all' || parseInt(managerSession.office) === 12) {
                officeList2 = JSON.stringify(cldOfficeConst.list);
            } else {
                const office = [];
                const officeIdList = await ctx.service.project.getOfficeList(managerSession.office, managerSession.userID);
                for (const oi of officeIdList) {
                    office[oi.office] = cldOfficeConst.list[oi.office];
                }
                officeList2 = JSON.stringify(office);
            }

            const renderData = {
                projectList,
                pageInfo,
                dropDownMenu,
                displayStatus: JSON.stringify(ctx.service.project.displayStatus),
                displayStatus2: ctx.service.project.displayStatus,
                statusClass: ctx.service.project.statusClass,
                officeList: cldOfficeConst.list,
                officeSelectList: officeList2,
            };
            await this.layout('project/index.ejs', renderData);
        }

        /**
         * 新增项目页面
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async add(ctx) {
            // 获取数据规则
            const rule = ctx.service.project.rule('project');
            const frontRule = ctx.helper.validateConvert(rule);

            //TODO 替换成获得cld人员
            const managerList = await ctx.service.manager.getManagerList('office', false);
            
            // 面包屑
            let breadCrumb = [
                { name: ctx.topPermission.name },
                { name: '项目版客户', url: ctx.controllerName },
                { name: '添加项目' },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
            const renderData = {
                rule: JSON.stringify(frontRule),
                officeList: JSON.stringify(cldOfficeConst.list),
                managerList: JSON.stringify(managerList),
                breadCrumb,
            };
            await this.layout('project/add.ejs', renderData);
        }

        /**
         * 修改项目页面
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async modify(ctx) {
            // 获取数据规则
            const rule = ctx.service.project.rule('projectModify');
            const frontRule = ctx.helper.validateConvert(rule);

            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                // 查找数据
                const projectData = await ctx.service.project.getDataById(id);
                const managerData = await ctx.service.manager.getDataById(projectData.creator);
                // 项目数据存入session
                ctx.session.sessionProjectData = projectData;
                const managerList = await ctx.service.manager.getManagerList('office', false);

                // 计价规范
                ctx.service.valuationList.searchFilter(ctx.request.query);
                const valuationList = await ctx.service.valuationList.getListWithBuilder();

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);
                const renderData = {
                    rule: JSON.stringify(frontRule),
                    projectData,
                    managerData,
                    officeList: JSON.stringify(cldOfficeConst.list),
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    managerList: JSON.stringify(managerList),
                    valuationList,
                    breadCrumb,
                };

                await this.layout('project/modify.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存项目操作
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async save(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id < 0) {
                    throw '参数错误';
                }
                // 获取数据规则
                const scene = id === 0 ? 'project' : 'projectModify';
                const rule = ctx.service.project.rule(scene);

                ctx.validate(rule);
   
                
                const result = await ctx.service.project.save(ctx.request.body, id);
                if (!result) {
                    throw '保存项目数据失败';
                }

                this.setMessage('保存项目数据成功', this.messageType.SUCCESS);
                ctx.redirect('/project');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除项目
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                const result = ctx.service.project.deleteProjectById(id);

                if (!result) {
                    throw '删除项目失败';
                }

                this.setMessage('删除项目成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 上传二维码照片
         * @param ctx
         * @returns {Promise<void>}
         */
        async uploadQrCode(ctx) {
            const responseData = {
                err: 0, msg: '', data: null,
            };
            try {
                const stream = await ctx.getFileStream();
                const create_time = Date.parse(new Date()) / 1000;
                const fileInfo = path.parse(stream.filename);
                const fileName = path.join('public/upload/qrcode', 'project_' + ctx.session.sessionProjectData.id.toString() + '_qrcode_' + create_time + fileInfo.ext);
                await ctx.helper.saveStreamFile(stream, path.join(this.app.baseDir, 'app', fileName));
                await sendToWormhole(stream);
                responseData.data = { base: fileInfo.base, path: fileName };
            } catch (err) {
                this.log(err);
                responseData.err = 1;
                responseData.msg = err;
            }
            ctx.body = responseData;
        }

        /**
         * 项目设置页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async setting(ctx) {

            // 获取数据规则
            const rule = ctx.service.project.rule('projectSetting');
            const frontRule = ctx.helper.validateConvert(rule);

            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                // 工程量清单列表
                ctx.service.stdGclList.searchFilter(ctx.request.query);
                const billList = await ctx.service.stdGclList.getListWithBuilder();

                // 项目节列表
                ctx.service.stdXmjList.searchFilter(ctx.request.query);
                const projectChapterList = await ctx.service.stdXmjList.getListWithBuilder();

                // 清单模板列表
                ctx.service.billsTemplateList.searchFilter(ctx.request.query);
                const standardList = await ctx.service.billsTemplateList.getListWithBuilder();

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);

                const renderData = {
                    rule: JSON.stringify(frontRule),
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    billList,
                    projectChapterList,
                    standardList: JSON.stringify(standardList),
                    breadCrumb,
                };
                await this.layout('project/setting.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存项目设置操作
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async saveSetting(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id < 0) {
                    throw '参数错误';
                }
                // 获取数据规则
                const rule = ctx.service.project.rule('projectSetting');
                ctx.validate(rule);

                const result = await ctx.service.project.saveSetting(ctx.request.body, id);

                if (!result) {
                    throw '保存项目数据失败';
                }

                this.setMessage('保存项目数据成功', this.messageType.SUCCESS);
                ctx.redirect('/project');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 项目标段页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async tender(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                const tenderList = await ctx.service.tender.getList();
                if (tenderList !== null) {
                    let i = 0;
                    for (const tender of tenderList) {
                        const projectAccountData = await ctx.service.projectAccount.getDataByCondition({ id: tender.user_id });
                        tenderList[i].username = projectAccountData.name;
                        i++;
                    }
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);

                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    tenderList,
                    breadCrumb,
                };
                await this.layout('project/tender.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 项目页面展示设置页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async pageshow(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);

                const renderData = {
                    projectData,
                    pageShowConst,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    breadCrumb,
                };
                await this.layout('project/page_show.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }
        /**
         * 项目页面展示设置修改
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async pageshowSet(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                const data = ctx.request.body;
                delete data._csrf;
                const result = await ctx.service.project.savePageshow(data, projectData.id);
                if (result) {
                    ctx.session.sessionProjectData.page_show = JSON.stringify(data);
                    this.setMessage('设置页面显示成功', this.messageType.SUCCESS);
                } else {
                    throw '设置页面显示失败';
                }
                ctx.redirect(ctx.request.header.referer);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 项目第三方接口设置页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async api2other(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);

                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    breadCrumb,
                };
                await this.layout('project/api2other.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 第三方接口设置修改
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async api2otherSave(ctx) {
            const projectData = ctx.session.sessionProjectData;
            const responseData = {
                err: 0, msg: '', data: {},
            };
            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                const postData = JSON.parse(ctx.request.body.data);
                if (postData.custom !== undefined) {
                    if (projectData.secret === null) {
                        postData.secret = ctx.helper.generateRandomString(20);
                        ctx.session.sessionProjectData.secret = postData.secret;
                    }
                    ctx.session.sessionProjectData.custom = parseInt(postData.custom);
                } else if (postData.can_api !== undefined) {
                    ctx.session.sessionProjectData.can_api = parseInt(postData.can_api);
                }
                const result = await ctx.service.project.update(postData, { id: projectData.id });
                if (result) {
                    responseData.data = ctx.session.sessionProjectData;
                    ctx.body = responseData;
                } else {
                    throw '入库失败';
                }
            } catch (error) {
                console.log(error);
                ctx.body = { err: 1, msg: error.toString(), data: null };
            }
        }

        /**
         * 项目办事处共享设置页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async office(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(projectData);

                const renderData = {
                    projectData,
                    status: ctx.service.project.displayStatus,
                    statusClass: ctx.service.project.statusClass,
                    // officeList: cldOfficeConst.list,
                    cldOfficeConst,
                    breadCrumb,
                };
                await this.layout('project/office.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 项目办事处共享修改
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async officeShare(ctx) {
            const projectData = ctx.session.sessionProjectData;

            try {
                // 查找数据
                if (projectData.id === undefined) {
                    throw '不存在对应的项目数据';
                }
                const result = await ctx.service.project.saveOfficeShare(ctx.request.body, projectData.id);
                if (result) {
                    ctx.session.sessionProjectData.office_share = ctx.request.body.office_share.join(',');
                    this.setMessage('设置办事处共享成功', this.messageType.SUCCESS);
                } else {
                    throw '设置办事处共享失败';
                }
                ctx.redirect(ctx.request.header.referer);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 检测工程建设项目是否容许创建
         * @param {*} ctx -egg 全局变量
         */
        async codeValid(ctx) {
            let result = false;
            const projectCode = ctx.request.query.code;

            try {
                if (projectCode === undefined || projectCode === '') {
                    throw '数据为空';
                }
                //工程建设项目编号不存在
                const projectCodeData = await ctx.service.project.findOne('code', projectCode);
                if (projectCodeData.length === 0) {
                    //TODO - 验证计量支付项目编号 -- 存在验证成功:容许创建
                    result = true;
                }
            } catch (error) {
                result = false;
            }
            ctx.body = result;
        }

        /**
         * 获取共用面包屑
         *
         * @param {Object} projectData - 项目数据
         * @return {String} - 返回面包屑
         */
        _getBreadCrumb(projectData) {
            // 面包屑
            const breadCrumb = [
                { name: this.ctx.topPermission.name },
                { name: '项目版客户', url: this.ctx.controllerName },
                { name: projectData.name },
            ];
            return this.ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
        }

    }

    return ProjectController;
};
