'use strict';

/**
 * 报表定制相关控制器
 *
 * @author Tony Kang
 * @date 2019/04/23
 * @version
 */

const COMMON_RPT_GRP_ID = 0;
const JpcEx = require('../reports/rpt_component/jpc_ex');
const JV = require('../reports/rpt_component/jpc_value_define');
const RPT_MAPPING_FIELDS = require('../const/report_fields');
const RPT_EXT_CODE = require('../const/report_ext_code');
const preDefinedRptDataAnalyzor = require('../lib/rpt_data_analysis');
const RPT_DEF_PROPERTIES = require('../const/report_defined_properties');


module.exports = app => {
    class ReportController extends app.BaseController {
        /**
         * 定制报表项目列表页
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            // console.log('report: 1');
            // 获取消息提示
            const message = this.ctx.session.message;
            // 取出后删除
            this.ctx.session.message = null;

            // const mappingFields = await ctx.service.rptMappingField.getMappingFields();
            const mappingFields = RPT_MAPPING_FIELDS;
            // console.log(mappingFields[0].items[10]);
            // const calcExtCode = await ctx.service.rptExtCodeTpl.getAllExtCodeTpl();
            const calcExtCode = RPT_EXT_CODE;
            // console.log('calcExtCode');
            // console.log(calcExtCode);
            const pid = ctx.request.query.pid;
            let treeNode = [];
            // console.log('pid: ' + pid);
            if (pid && pid !== '') {
                treeNode = await ctx.service.rptTreeNode.getNodesByProjectId([pid]);
            }
            const pCode = ctx.request.query.code;
            const pName = ctx.request.query.name;

            const defined_handle_options = preDefinedRptDataAnalyzor.analysisDefine;

            const renderData = {
                message: JSON.stringify(message),
                isCommon: (pid === -1 || pid === '-1'),
                project_info: pCode + '-' + pName,
                cmn_rpt_data: JSON.stringify(treeNode),
                cmn_rpt_mapping_fields: JSON.stringify(mappingFields),
                calc_ext_code: JSON.stringify(calcExtCode),
                // cmn_rpt_data: treeNode,
                defined_handle_options: JSON.stringify(defined_handle_options),
            };
            // await this.layout('report/index.ejs', renderData); // 用this.layout方法会渲染统一界面风格
            await this.ctx.render('report/index.ejs', renderData); // 这种渲染是完全从头开始渲染
        }

        /**
         * 保存定制报表顶节点数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async save(ctx) {
            try {
                // console.log(ctx.request.body.params);
                const params = JSON.parse(ctx.request.body.params),
                    doc = params.doc;
                doc.items = JSON.stringify(doc.items);
                const result = await ctx.service.rptTreeNode.updateTopNode(doc);
                if (!result) {
                    throw '保存失败';
                }
                // this.setMessage('保存成功', this.messageType.SUCCESS);
                ctx.body = {};
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                // ctx.redirect(ctx.request.header.referer);
            }
        }

        async partialUpdateTreeNode(ctx) {
            // 这个是局部刷新，原理是根据topNodeID找到后台的topNode，根据路径(有多个路径，可以实现多个子节点同时刷新)来替换（nodeArray里的）后台数据库的相关节点
            // 这里需要一个调整，就是新增目录及模板需要加给后缀(new(Date)).getTime() ，以保证不重叠(真重叠了算倒霉)
            // pathArray的path描述：{ operation_type: '', // 操作类型：‘update’ ‘add’、‘delete’
            //                       node_path: [],      // 节点路径：如：['02.广东', '03.增城北绕线项目定制报表']，表示Top节点的items下的那个‘02.广东’子节点(要full scan)的items下的 ‘03.增城北绕线项目定制报表’子节点
            //                     }
            // nodeArray： 与pathArray一一对应
            try {
                const params = JSON.parse(ctx.request.body.params);
                // console.log(params);
                const topNodeID = params.topNodeID;
                const pathArray = params.pathArray;
                const nodeArray = params.nodeArray;
                const result = await ctx.service.rptTreeNode.getNodeById(topNodeID);
                if (result && result.length === 1) {
                    const topNodeItems = JSON.parse(result[0].items);
                    const _getCurrentNodeSerialOrder = function(parentItems, pathName) {
                        let rst = -1;
                        if (parentItems) {
                            for (let idx = 0; idx < parentItems.length; idx++) {
                                if (parentItems[idx].name === pathName) {
                                    rst = idx;
                                    break;
                                }
                            }
                        }
                        return rst;
                    };

                    const _updateNodeByPath = function(path, node) {
                        let rst = false;
                        let tmpParentItems = topNodeItems;
                        for (let idx = 0; idx < path.node_path.length; idx++) {
                            const nIdx = _getCurrentNodeSerialOrder(tmpParentItems, path.node_path[idx]);
                            if (nIdx >= 0) {
                                if (idx === path.node_path.length - 1) {
                                    tmpParentItems[nIdx] = node;
                                    rst = true;
                                } else {
                                    tmpParentItems = tmpParentItems[nIdx].items;
                                }
                            }
                        }
                        return rst;
                    };
                    const _addNodeByPath = function(path, node) {
                        let rst = false;
                        // console.log('node:');
                        // console.log(node);
                        let tmpParentItems = topNodeItems;
                        if (path === null || path === '' || path.node_path.length === 0) {
                            topNodeItems.push(node);
                            rst = true;
                        } else {
                            for (let idx = 0; idx < path.node_path.length; idx++) {
                                const nIdx = _getCurrentNodeSerialOrder(tmpParentItems, path.node_path[idx]);
                                if (nIdx >= 0) {
                                    if (idx === path.node_path.length - 1) {
                                        // console.log('tmpParentItems[nIdx]:');
                                        // console.log(tmpParentItems[nIdx]);
                                        if (tmpParentItems[nIdx].nodeType === 1) {
                                            if (!tmpParentItems[nIdx].items) {
                                                tmpParentItems[nIdx].items = [];
                                            }
                                            tmpParentItems[nIdx].items.push(node);
                                            rst = true;
                                        }
                                    } else {
                                        tmpParentItems = tmpParentItems[nIdx].items;
                                    }
                                }
                            }
                        }
                        return rst;
                    };
                    const _deleteNodeByPath = function(path, node) {
                        let rst = false;
                        let tmpParentItems = topNodeItems;
                        for (let idx = 0; idx < path.node_path.length; idx++) {
                            const nIdx = _getCurrentNodeSerialOrder(tmpParentItems, path.node_path[idx]);
                            if (nIdx >= 0) {
                                if (idx === path.node_path.length - 1) {
                                    // tmpParentItems[nIdx] = node;
                                    tmpParentItems.splice(nIdx, 1);
                                    rst = true;
                                } else {
                                    tmpParentItems = tmpParentItems[nIdx].items;
                                }
                            }
                        }
                        return rst;
                    };
                    if (pathArray && pathArray.length > 0 && nodeArray && nodeArray.length === pathArray.length) {
                        for (let idx = 0; idx < pathArray.length; idx++) {
                            if (pathArray[idx].operation_type === 'update') {
                                _updateNodeByPath(pathArray[idx], nodeArray[idx]);
                            } else if (pathArray[idx].operation_type === 'add') {
                                _addNodeByPath(pathArray[idx], nodeArray[idx]);
                            } else if (pathArray[idx].operation_type === 'delete') {
                                _deleteNodeByPath(pathArray[idx], nodeArray[idx]);
                            } else {
                                // out of control
                            }
                        }
                    }
                    try {
                        const doc = result[0];
                        doc.items = JSON.stringify(topNodeItems);
                        const updateRst = await ctx.service.rptTreeNode.updateTopNode(doc);
                        // console.log(updateRst);
                        if (!updateRst) {
                            throw '更新失败';
                        }
                    } catch (ex) {
                        console.log(ex);
                    }
                    ctx.body = {};
                    ctx.status = 201;
                } else {
                    ctx.body = { msg: 'no result was found!' };
                    ctx.status = 201;
                }
                // this.setMessage('保存成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                // ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * copy报表模板
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async updateDetailNode(ctx) {
            try {
                const params = JSON.parse(ctx.request.body.params);
                // console.log(params);
                const result = await ctx.service.rptTreeNode.getNodeById(params.topNodeID);
                if (result && result.length === 1) {
                    const _updateNode = function(nodeItem) {
                        let rst = false;
                        try {
                            if (nodeItem.ID === params.ID && nodeItem.name === params.name && nodeItem.refId === params.refId) {
                                rst = true;
                                if (!nodeItem.hasOwnProperty('flags')) {
                                    nodeItem.flags = {};
                                }
                                nodeItem.flags[params.flagProp] = params.flagPropValue;
                                // console.log('changed node: ');
                                // console.log(nodeItem);
                            }
                            if (!rst && nodeItem.items && nodeItem.items.length > 0) {
                                for (const subItem of nodeItem.items) {
                                    rst = _updateNode(subItem);
                                    if (rst) break;
                                }
                            }
                        } catch (ex) {
                            rst = false;
                            console.log(ex);
                        }
                        return rst;
                    };
                    const tmpNodes = JSON.parse(result[0].items);
                    // console.log(tmpNodes);
                    // let hasChanged = false;
                    for (const nItem of tmpNodes) {
                        if (_updateNode(nItem)) {
                            const doc = result[0];
                            doc.items = JSON.stringify(tmpNodes);
                            try {
                                const updateRst = await ctx.service.rptTreeNode.updateTopNode(doc);
                                if (!updateRst) {
                                    throw '更新失败';
                                }
                            } catch (ex) {
                                console.log(ex);
                            }
                            break;
                        }
                    }
                    ctx.body = {};
                    ctx.status = 201;
                } else {
                    ctx.body = { msg: 'no result was found!' };
                    ctx.status = 201;
                }
                // this.setMessage('保存成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                // ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * copy报表模板
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async copyNodes(ctx) {
            try {
                const params = JSON.parse(ctx.request.body.params),
                    doc = params.doc;
                doc.items = JSON.stringify(doc.items);
                const result = await ctx.service.rptTreeNode.updateTopNode(doc);
                if (!result) {
                    throw '保存失败';
                }
                // this.setMessage('保存成功', this.messageType.SUCCESS);
                ctx.body = {};
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                // ctx.redirect(ctx.request.header.referer);
            }
        }
        /**
         * 保存定制报表顶节点数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async createReportTemplate(ctx) {
            try {
                // console.log(ctx.request.body.params);
                const params = JSON.parse(ctx.request.body.params);
                let refDemoId = 4; // 默认流水式
                if (params.rptDftTplId === 5) {
                    // 交叉式
                    refDemoId = 5;
                } else if (params.rptDftTplId === 6) {
                    // 账单式
                    refDemoId = 6;
                }
                const demoTplData = await ctx.service.rptTpl.getTplById(refDemoId);
                if (!demoTplData || demoTplData.length !== 1) {
                    throw '创建模板失败';
                }
                demoTplData[0].rpt_tpl_name = params.rptName;
                const rst = await ctx.service.rptTpl.copyAndCreateTemplate(demoTplData[0]);
                if (!rst || rst.affectedRows !== 1) {
                    throw '创建模板失败';
                }
                const rstTplData = await ctx.service.rptTpl.getTplById(rst.insertId);
                ctx.body = { data: rstTplData[0] };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        /**
         * 创建定制报表组（顶节点）数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async createNewRptGrp(ctx) {
            try {
                // console.log(ctx.request.body.params);
                const params = JSON.parse(ctx.request.body.params);
                const rst = await ctx.service.rptTreeNode.createNode(params.pid, params.name);
                if (!rst || rst.affectedRows !== 1) {
                    throw '创建报表组失败';
                }
                const newRptGrp = await ctx.service.rptTreeNode.getNodeById(rst.insertId);
                const newData = {
                    id: rst.insertId,
                    pid: newRptGrp[0].pid,
                    name: newRptGrp[0].name,
                    rptAmt: 0,
                    last_update_time: newRptGrp[0].last_update_time,
                }
                ctx.body = { data: newData };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        /**
         * 修改报表组（顶节点）数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async updateRptGrp(ctx) {
            try {
                const params = JSON.parse(ctx.request.body.params);
                const rst = await ctx.service.rptTreeNode.update({name: params.name},{id: params.id});
                if (!rst) {
                    throw '修改报表组失败';
                }
                ctx.body = { data: rst };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        async deleteRptGrp(ctx) {
            try {
                // console.log(ctx.request.body.params);
                const params = JSON.parse(ctx.request.body.params);
                const rptGrp = await ctx.service.rptTreeNode.getNodeById(params.id);
                if (!rptGrp || (rptGrp[0].items && JSON.parse(rptGrp[0].items).length !== 0)) {
                    throw '该报表组不存在或该报表组存在报表不可删';
                }
                const rst = await ctx.service.rptTreeNode.deleteById(params.id);
                if (!rst) {
                    throw '删除报表组失败';
                }

                ctx.body = { data: rst };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        /**
         * 获取报表组（顶节点）数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async getRptGrp(ctx) {
            try {
                // console.log('in getRptGrp function.');
                const params = JSON.parse(ctx.request.body.params);
                // console.log(params);
                const rptGrp = await ctx.service.rptTreeNode.getNodesByProjectId([params.pid]);
                // console.log(rptGrp);
                const getItmesAmt = function(topItems) {
                    let rst = 0;
                    if (topItems && topItems.length > 0) {
                        // rst = topItems.length;
                        for (const subItem of topItems) {
                            if (subItem.nodeType === 2) {
                                rst++;
                            } else {
                                rst = rst + getItmesAmt(subItem.items);
                            }
                        }
                    }
                    return rst;
                };
                const rst = [];
                for (const rptNode of rptGrp) {
                    const amt = getItmesAmt(JSON.parse(rptNode.items));
                    rst.push({ id: rptNode.id, pid: params.pid, name: rptNode.name, rptAmt: amt, last_update_time: rptNode.last_update_time });
                }
                // console.log(rst);
                ctx.body = { data: rst };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        /**
         * 获取报表模板对象
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async getReportTemplate(ctx) {
            try {
                // console.log(ctx.request.body.params);
                const params = JSON.parse(ctx.request.body.params);
                const rstTplData = await ctx.service.rptTpl.getTplById(params.rptTplId);
                // console.log(rstTplData);
                if (!rstTplData || rstTplData.length !== 1) {
                    throw '获取模板失败';
                }
                ctx.body = { data: rstTplData[0] };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        async updateRptTpl(ctx) {
            try {
                // console.log(ctx.request.body.params);
                const params = JSON.parse(ctx.request.body.params);
                const rstTplData = await ctx.service.rptTpl.updateRptTpl(params.tplId, params.tplName, params.rptTpl, params.cid);
                // this.setMessage(params.tplName + ' 保存成功', this.messageType.SUCCESS);
                const msg = this.createMsg(params.tplName + ' 保存成功', this.messageType.SUCCESS);
                ctx.body = { data: rstTplData[0], message: msg };
                ctx.status = 201;
            } catch (ex) {
                this.setMessage(ex.toString(), this.messageType.ERROR);
            }

        }

        /**
         * 获取报表预定义好的配置(字体、边框、输出控制)
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async getPreDefineCfg(ctx) {
            try {
                // const cfgData = await ctx.service.rptPreDefineCfg.getCfgById('Administrator');
                // console.log(cfgData);
                // if (!cfgData || cfgData.length !== 1) {
                //     throw '获取预定义配置失败';
                // }
                // ctx.body = { data: cfgData[0].defined_content };
                const cfgData = RPT_DEF_PROPERTIES;
                ctx.body = { data: JSON.stringify(cfgData) };
                ctx.status = 201;
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
        }

        async getPreviewPage(ctx) {
            try {
                const params = JSON.parse(ctx.request.body.params);
                const pageSize = params.pageSize;
                let rptTpl = await ctx.service.rptTpl.getTplById(params.rpt_tpl_id);
                if (!rptTpl || rptTpl.length !== 1) {
                    throw '获取模板失败';
                }
                const printCom = JpcEx.createNew();
                // const defProperties = await ctx.service.rptPreDefineCfg.getCfgById('Administrator');
                const defProperties = RPT_DEF_PROPERTIES;
                // JSON.parse(defProperties[0].defined_content);
                rptTpl = JSON.parse(rptTpl[0].rpt_content);
                rptTpl[JV.NODE_MAIN_INFO][JV.NODE_PAGE_INFO][JV.PROP_PAGE_SIZE] = pageSize;
                setupSomeDftCustomizeCfg(rptTpl);
                printCom.initialize(rptTpl);
                // const pageRst = printCom.outputAsPreviewPage(rptTpl, JSON.parse(defProperties[0].defined_content));
                const pageRst = printCom.outputAsPreviewPage(rptTpl, defProperties);
                // console.log(pageRst);
                ctx.body = { data: pageRst };
                ctx.status = 201;
            } catch (ex) {
                this.setMessage(ex.toString(), this.messageType.ERROR);
            }
        }

        createMsg(msg, type) {
            const rst = {};
            rst.message = msg;
            rst.type = type;
            switch (type) {
                case this.messageType.SUCCESS:
                    rst.icon = 'check';
                    break;
                case this.messageType.ERROR:
                    rst.icon = 'exclamation-circle';
                    break;
                case this.messageType.INFO:
                    rst.icon = 'info-circle';
                    break;
                case this.messageType.WARNING:
                    rst.icon = 'warning';
                    break;
                default:
                    break;
            }
            return rst;
        }
    }

    return ReportController;
};

function setupSomeDftCustomizeCfg(rptTpl) {
    rptTpl[JV.NODE_MAIN_INFO][JV.NODE_MARGINS][JV.PROP_LEFT] = 1.2;
    rptTpl[JV.NODE_MAIN_INFO][JV.NODE_MARGINS][JV.PROP_RIGHT] = 1.2;
};
