'use strict';

/**
 * Created by Tony on 2019/6/24.
 */

const cldOfficeConst = require('../const/cld_office');

module.exports = app => {
    class ReportProjectController extends app.BaseController {
        /**
         * 定制报表项目列表页
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;
            const query = ctx.request.query;
            const rpt_prj_List = [];
            const filter = {};
            let office = -1;
            const managerSession = ctx.session.managerSession;
            // 判断用户权限和所属办事处（PS:只有总部非造价研究中心和超级管理员可查看所有管理员用户，其它只显示该分组用户）
            let officeList = [];
            const permission = managerSession.permission;
            if (permission === 'all' || parseInt(managerSession.office) === cldOfficeConst.type.ZONGBU) {
                officeList = JSON.stringify(cldOfficeConst.list);
            } else {
                const offices = [];
                const officeIdList = await ctx.service.project.getOfficeList(managerSession.office, managerSession.userID);
                office = [];
                for (const oi of officeIdList) {
                    offices[oi.office] = cldOfficeConst.list[oi.office];
                    office.push(oi.office);
                }
                office = office.length > 0 ? office : -1;
                officeList = JSON.stringify(offices);
            }

            if (query.hasOwnProperty('office') && query.office !== '') {
                office = query.office;
            }

            // console.log(1);
            // filter.status = 2;
            // filter.managerIds = [];
            // if (query.hasOwnProperty('office') && query.office !== '') {
            //     office = query.office;
            //     if (query.select_sale_responsor !== '') {
            //         // 找项目
            //         filter.managerIds.push(query.select_sale_responsor);
            //     } else {
            //         // filter.managerIds.push(-1);
            //     }
            //     filter.office = office;
            //     filter.name = query.keyword;
            //     // ctx.service.project.searchFilterByManager(filter);
            // } else {
            //     office = -1;
            //     filter.office = managerSession.office !== cldOfficeConst.type.ZONGBU ? managerSession.office : '';
            // }
            ctx.service.project.searchFilterByManager(ctx.request.query, managerSession);
            const total = await ctx.service.project.getCountWithBuilder();
            const projectList = await ctx.service.project.getListWithBuilder();
            const ppl = await getResponsePeopleByOffice(ctx, office);
            const prjsArr = [];
            for (const prj of projectList) {
                prjsArr.push(prj.id);
            }
            const rptGroups = [];
            if (prjsArr.length > 0) {
                const rptNodes = await ctx.service.rptTreeNode.getNodesByProjectId(prjsArr);
                const getItmesAmt = function(topItems) {
                    let rst = 0;
                    if (topItems && topItems.length > 0) {
                        // rst = topItems.length;
                        for (const subItem of topItems) {
                            if (subItem.nodeType === 2) {
                                rst++;
                            } else {
                                rst = rst + getItmesAmt(subItem.items);
                            }
                        }
                    }
                    return rst;
                };
                for (const prj of projectList) {
                    let response_name = '';
                    const manager = await ctx.service.manager.getDataById(prj.manager_id);
                    response_name = manager !== null ? manager.username : '';
                    let group = 0;
                    let pages = 0;
                    for (const node of rptNodes) {
                        if (node.pid === prj.id) {
                            group++;
                            const items = JSON.parse(node.items);
                            const curAmt = getItmesAmt(items);
                            pages += curAmt;
                            rptGroups.push({ id: node.id, pid: prj.id, name: node.name, rptAmt: curAmt, last_update_time: node.last_update_time });
                        }
                    }
                    rpt_prj_List.push({ project_id: prj.id, project_code: prj.code, project_name: prj.name, office_name: cldOfficeConst.list[prj.office], rpt_grp: group + '组 / ' + pages + '张', response_people: response_name });
                }
            }
            // console.log(5);
            let dropDownMenu = [];
            if (managerSession.office === cldOfficeConst.type.ZONGBU || managerSession.permission === 'all') {
                dropDownMenu = [
                    { name: '编辑通用报表', url: 'report/modifyReport?pid=-1', isNewPage: 'T' },
                    // { name: '添加项目', url: 'addpoj', isPopup: 'T' },
                ];
            }
            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };
            const renderData = {
                rpt_prj_List,
                pageInfo,
                dropDownMenu,
                officeList,
                officePeople: JSON.stringify(ppl),
                rptNodes: JSON.stringify(rptGroups),
            };
            await this.layout('report/indexRptPrj.ejs', renderData);
        }

        async getOfficeResponsePeople(ctx) {
            // console.log('in getOfficeResponsePeople');
            const params = JSON.parse(ctx.request.body.params);
            const rst = await getResponsePeopleByOffice(ctx, params.office);
            // console.log(rst);
            ctx.body = { data: rst };
            ctx.status = 201;
        }

        createMsg(msg, type) {
            const rst = {};
            rst.message = msg;
            rst.type = type;
            switch (type) {
                case this.messageType.SUCCESS:
                    rst.icon = 'check';
                    break;
                case this.messageType.ERROR:
                    rst.icon = 'exclamation-circle';
                    break;
                case this.messageType.INFO:
                    rst.icon = 'info-circle';
                    break;
                case this.messageType.WARNING:
                    rst.icon = 'warning';
                    break;
                default:
                    break;
            }
            return rst;
        }
    }

    return ReportProjectController;
};

async function getResponsePeopleByOffice(ctx, office) {
    // const queryKey = {};
    const searchSession = {
        columns: ['id', 'username', 'real_name', 'create_time', 'last_login', 'login_ip',
            'group_id', 'token', 'can_login'],
    };
    office !== -1 ? searchSession.where = { office } : '';
    // const searchSession = ctx.session.managerSession;
    // queryKey.type = 1;
    // searchSession.office = office;
    // ctx.service.manager.searchFilter(queryKey, searchSession); // 依据manager service的参数逻辑，根据实际情况来造过滤参数
    const managerList = await ctx.service.manager.getAllDataByCondition(searchSession);
    const rst = [];
    // rst.push({ id: -1, name: '所有人员' });
    for (let idx = 0; idx < managerList.length; idx++) {
        const manager = managerList[idx];
        rst.push({ id: manager.id, name: manager.username });
    }
    return rst;
}
