'use strict';

/**
 * 计价标准相关控制器
 *
 * @author EllisRan
 * @date 2019/9/16
 * @version
 */
module.exports = app => {
    class ValuationController extends app.BaseController {
        /**
         * 列表页
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;

            // 获取计价标准数据
            ctx.service.valuationList.searchFilter(ctx.request.query);
            const valuationList = await ctx.service.valuationList.getListWithBuilder();
            const total = await ctx.service.valuationList.count();

            // 获取已选计价标准的项目信息
            const projectList = await ctx.service.project.getAllDataByCondition();
            let check_array = [];
            for (const project of projectList) {
                if (project.valuation !== '' && project.valuation !== null) {
                    check_array = check_array.concat(project.valuation.split(','));
                }
            }

            for (const index in valuationList) {
                valuationList[index].canDel = check_array.indexOf(valuationList[index].id.toString()) === -1;
            }

            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / ctx.app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            const dropDownMenu = [
                { name: '添加新标准', url: ctx.controllerName + '/add/0' },
            ];

            const renderData = {
                valuationList,
                pageInfo,
                dropDownMenu,
            };
            await this.layout('valuation/index.ejs', renderData);
        }

        /**
         * 新增
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async add(ctx) {
            const rule = ctx.service.valuationList.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();

            const valuationListData = {};
            const renderData = {
                jsValidator,
                valuationListData,
            };
            await this.layout('valuation/save.ejs', renderData);
        }

        /**
         * 保存列表数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async save(ctx) {
            try {
                const id = ctx.params.id;
                const rule = ctx.service.valuationList.rule();
                ctx.validate(rule);
                const result = await ctx.service.valuationList.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                ctx.redirect('/' + ctx.controllerName);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);
            try {
                if (isNaN(id) || id <= 0) {
                    throw 'id错误';
                }
                const result = await ctx.service.valuationList.deleteValuation(id);
                if (!result) {
                    throw '删除失败';
                }

                this.setMessage('删除成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 详情页面
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async detail(ctx) {
            let id = ctx.params.id;
            try {
                id = parseInt(id);
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                const valuationInfo = await ctx.service.valuationList.getDataById(id);
                // 工程量清单列表
                const billList = await ctx.service.stdGclList.getAllDataByCondition();

                // 项目节列表
                const projectChapterList = await ctx.service.stdXmjList.getAllDataByCondition();

                // 清单模板列表
                const standardList = await ctx.service.billsTemplateList.getAllDataByCondition();

                const rule = ctx.service.valuationList.rule();
                const jsValidator = await this.jsValidator.convert(rule).build();

                const renderData = {
                    jsValidator,
                    valuationInfo,
                    billList,
                    projectChapterList,
                    standardList,
                    billList2: JSON.stringify(billList),
                    projectChapterList2: JSON.stringify(projectChapterList),
                    standardList2: JSON.stringify(standardList),
                };
                await this.layout('valuation/detail.ejs', renderData, 'valuation/detail_modal.ejs');

            } catch (error) {
                ctx.redirect('/' + ctx.controllerName);
            }
        }

        /**
         * 保存计量标准数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async detailSave(ctx) {
            try {
                const responseData = {
                    err: 0, msg: '', data: {},
                };
                const id = ctx.params.id;
                const postData = JSON.parse(ctx.request.body.data);
                const data = {};
                if (postData.name !== undefined && postData.name !== null && postData.name !== '') {
                    data.name = postData.name;
                } else {
                    const type = postData.type;
                    switch (type) {
                        case 'chapter_id':
                            data.chapter_id = postData.list;
                            break;
                        case 'bill_id':
                            data.bill_id = postData.list;
                            break;
                        case 'template_id':
                            data.template_id = postData.list;
                            break;
                        case 'list_chapter_id':
                            data.list_chapter_id = postData.list;
                            break;
                        case 'list_bill_id':
                            data.list_bill_id = postData.list;
                            break;
                        case 'list_template_id':
                            data.list_template_id = postData.list;
                            break;
                        default: throw '参数有误';
                    }
                }
                const result = await ctx.service.valuationList.save(data, id);
                if (result) {
                    ctx.body = responseData;
                } else {
                    throw '添加失败';
                }
            } catch (error) {
                console.log(error);
                ctx.body = { err: 1, msg: error.toString(), data: null };
            }
        }
    }

    return ValuationController;
};
