'use strict';

/**
 * 版本管理控制器
 *
 * @author CaiAoLin
 * @date 2017/10/25
 * @version
 */

module.exports = app => {
    class VersionController extends app.BaseController {

        /**
         * 消息列表
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;

            // 筛选数据
            ctx.sort = ['id', 'DESC'];
            ctx.service.version.searchFilter(ctx.request.query);
            // 获取数据
            const versionList = await ctx.service.version.getListWithBuilder();
            const total = await ctx.service.version.count();

            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            const dropDownMenu = [
                { name: '添加新版本', url: ctx.controllerName + '/add/0' },
            ];

            const renderData = {
                // versionList,
                versionList: JSON.parse(JSON.stringify(versionList).replace(/\\r\\n/g, '<br>').replace(/\\"/g, '&#34').replace(/'/g, '&#39')),
                pageInfo,
                dropDownMenu,
            };
            await this.layout('version/index.ejs', renderData, 'version/modal.ejs');
        }

        /**
         * 新增
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async add(ctx) {
            const rule = ctx.service.version.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();

            const id = ctx.params.id;
            const versionData = parseInt(id) === 0 ? {} : await ctx.service.version.getDataById(id);
            let breadCrumb = [
                { name: ctx.topPermission.name },
                { name: '升级说明', url: ctx.controllerName },
                { name: parseInt(id) === 0 ? '添加版本' : '修改版本' },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
            const renderData = {
                jsValidator,
                versionData,
                breadCrumb,
            };
            await this.layout('version/save.ejs', renderData);
        }

        /**
         * 保存列表数据
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async save(ctx) {
            try {
                const id = ctx.params.id;
                const rule = ctx.service.version.rule();
                ctx.validate(rule);
                const result = await ctx.service.version.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                ctx.redirect('/' + ctx.controllerName);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);
            try {
                if (isNaN(id) || id <= 0) {
                    throw 'id错误';
                }
                const result = await ctx.service.version.deleteById(id);
                if (!result) {
                    throw '删除失败';
                }

                this.setMessage('删除成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }

    }

    return VersionController;
};
