'use strict';

/**
 * 白名单控制器
 *
 * @author CaiAoLin
 * @date 2017/10/16
 * @version
 */

module.exports = app => {

    class WhiteListController extends app.BaseController {

        /**
         * 白名单列表页
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async index(ctx) {
            // 筛选数据
            ctx.service.whiteList.searchFilter(ctx.request.query);
            const whiteListData = await ctx.service.whiteList.getListWithBuilder();
            const total = await ctx.service.whiteList.count();

            // 分页相关
            const pageInfo = {
                page: ctx.page,
                total: parseInt(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            const dropDownMenu = [
                { name: '新增白名单', url: ctx.controllerName + '/add' },
            ];

            const renderData = {
                whiteListData,
                pageInfo,
                type: ctx.service.whiteList.typeDisplay,
                whiteListType: JSON.stringify(ctx.service.whiteList.typeDisplay),
                dropDownMenu,
            };
            await this.layout('white_list/index.ejs', renderData);
        }

        /**
         * 新增白名单页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async add(ctx) {
            // 获取验证规则
            const rule = ctx.service.whiteList.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();

            let breadCrumb = [
                { name: '白名单列表', url: ctx.controllerName },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
            const renderData = {
                jsValidator,
                whiteListType: JSON.stringify(ctx.service.whiteList.typeDisplay),
                whiteListData: [],
                breadCrumb,
            };

            await this.layout('white_list/save.ejs', renderData);
        }

        /**
         * 修改白名单
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async modify(ctx) {
            try {
                let id = ctx.params.id;
                id = parseInt(id);

                // 获取验证规则
                const rule = ctx.service.whiteList.rule();
                const jsValidator = await this.jsValidator.convert(rule).build();

                // 查找对应数据
                const whiteListData = await ctx.service.whiteList.getDataById(id);

                let breadCrumb = [
                    { name: '白名单列表', url: ctx.controllerName },
                ];
                breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
                const renderData = {
                    jsValidator,
                    whiteListType: JSON.stringify(ctx.service.whiteList.typeDisplay),
                    whiteListData,
                    breadCrumb,
                };
                await this.layout('white_list/save.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存白名单数据
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async save(ctx) {
            try {
                const id = ctx.params.id;

                // 获取验证规则
                const rule = ctx.service.whiteList.rule();
                ctx.validate(rule);

                const result = await ctx.service.whiteList.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                ctx.redirect('/white-list');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }

        }

        /**
         * 删除白名单数据
         * @param {Object} ctx - 全局上下文
         */
        async delete(ctx) {
            const id = ctx.params.id;
            try {
                await ctx.service.whiteList.delById(id);
                ctx.redirect('/white-list');
            } catch (error) {
                this.log(error.toString());
                ctx.redirect('/white-list');
                throw error;
            }
        }

    }

    return WhiteListController;
};
