'use strict';

/**
 * CLD相关接口
 *
 * @author CaiAoLin
 * @date 2017/10/18
 * @version
 */

// 加密类
const crypto = require('crypto');
class Cld {

    /**
     * 构造函数
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    constructor(ctx) {
        this.authUrl = 'http://cld.smartcost.com.cn/api/auth';
        this.getDataUrl = 'http://cld.smartcost.com.cn/api/getData';
        this.app = 'scConstruct';
        this.token = 'sc@ConS!tru@ct*88';
        this.ctx = ctx;
    }

    /**
     * CLD登录验证
     *
     * @param {String} username - cld用户名
     * @param {String} password - cld密码
     * @return {boolean} - 验证结果
     */
    async loginValid(username, password) {
        let result = false;
        try {
            if (username === '' || password === '') {
                throw '用户名或密码错误';
            }
            // 生成加密token
            const [encryptToken, postTime] = this.generateCLDToken();
            const postData = {
                username,
                password,
                time: postTime,
                token: encryptToken,
                app: this.app,
            };

            const responseData = await this.ctx.helper.sendRequest(this.authUrl, postData);
            if (responseData.err !== 0) {
                throw '接口返回错误:' + responseData.err;
            }
            
            // 如果验证成功，则新增CLD数据到数据库
            if (responseData.data !== '') {
                const addResult = await this.ctx.service.manager.addCldUser(responseData.data);
                if (!addResult) {
                    console.log('cld user add error');
                }
            }
            result = true;
        } catch (error) {
            console.log('cld:' + error);
            result = false;
        }

        return result;
    }

    /**
     * 生成CLD Token
     *
     * @return {Array} - 生成的token，当前生成的时间
     */
    generateCLDToken() {
        let currentTime = new Date().getTime();
        currentTime = parseFloat(currentTime / 1000).toFixed(0);

        const encryptToken = crypto.createHmac('sha1', this.token).update(this.token + currentTime)
            .digest().toString('base64');

        return [encryptToken, currentTime];
    }

    /**
     * 获取CLD用户数据
     *
     * @param {Number} cldID - cld中的id
     * @return {String} - 返回json数据
     */
    async getCldUserData(cldID) {
        let result = {};
        try {
            result = cldID;
        } catch (error) {
            result = {};
        }

        return result;
    }

}

module.exports = Cld;
