$(document).ready(() => {
    let modified = false;
    const iTypeCol = dpSpreadSetting.cols.findIndex(function (x) {
        return x.field === 'ptype';
    });
    const iMinusCol = dpSpreadSetting.cols.findIndex(function (x) {
        return x.field === 'minus';
    });    

    const dpSpread = SpreadJsObj.createNewSpread($('#dealPaySpread')[0]);
    const dpSheet = dpSpread.getActiveSheet();
    SpreadJsObj.initSheet(dpSheet, dpSpreadSetting);

    SpreadJsObj.loadSheetData(dpSheet, SpreadJsObj.DataType.Data, dealpay);

    const refreshActnValid = function () {
        const setObjEnable = function (obj, enable) {
            if (enable) {
                obj.removeClass('disabled').removeClass('text-dark');
            } else {
                obj.addClass('disabled').addClass('text-dark');
            }
        };
        const sels = dpSheet.getSelections();
        if (sels && sels[0]) {
            const curRow = sels[0].row;
            const ptype = dpSheet.getCell(curRow, iTypeCol).value();
            setObjEnable($('#add'), curRow >= 2);
            setObjEnable($('#del'), ptype === payType.normal);
            setObjEnable($('#up-move'), curRow > 3);
            setObjEnable($('#down-move'), curRow > 2 && curRow < dpSheet.getRowCount() - 1);
        } else {
            setObjEnable($('#add'), false);
            setObjEnable($('#del'), false);
            setObjEnable($('#up-move'), false);
            setObjEnable($('#down-move'), false);
        }
    };
    dpSheet.bind(spreadNS.Events.SelectionChanged, function (e, info) {
        refreshActnValid();
    });
    refreshActnValid();

    $('#add').click(function() {
        const sels = dpSheet.getSelections();
        const curRow = sels && sels[0] ? sels[0].row + 1 : dpSheet.getRowCount();
        dpSheet.addRows(curRow, 1);
        dpSheet.getCell(curRow, iTypeCol).value(payType.normal).locked(true).hAlign(1);
        dpSheet.getCell(curRow, iMinusCol).hAlign(1);
        if (sels && sels[0]) {
            dpSheet.setSelection(curRow, sels[0].col, sels[0].rowCount, sels[0].colCount);
        } else {
            dpSheet.setSelection(curRow, 1, 0, 0);
        }
        refreshActnValid();
    });

    $('#del').click(function() {
        const sels = dpSheet.getSelections();
        const curRow = sels && sels[0] ? sels[0].row : dpSheet.getRowCount() - 1;
        dpSheet.deleteRows(curRow, 1);
        refreshActnValid();
    });

    $('#up-move').click(function () {
        const sels = dpSheet.getSelections();
        const curRow = sels && sels[0] ? sels[0].row : dpSheet.getRowCount() - 1;
        SpreadJsObj.beginMassOperation(dpSheet);
        dpSheet.addRows(dpSheet.getRowCount(), 1);
        dpSheet.moveTo(curRow - 1, -1, dpSheet.getRowCount() - 1, -1, 1, -1, GC.Spread.Sheets.CopyToOptions.value);
        dpSheet.moveTo(curRow, -1, curRow - 1, -1, 1, -1, GC.Spread.Sheets.CopyToOptions.value);
        dpSheet.moveTo(dpSheet.getRowCount() - 1, -1, curRow, -1, 1, -1, GC.Spread.Sheets.CopyToOptions.value);
        dpSheet.deleteRows(dpSheet.getRowCount() - 1, 1);
        SpreadJsObj.endMassOperation(dpSheet);
        dpSheet.setSelection(curRow - 1, sels[0].col, sels[0].rowCount, sels[0].colCount);
        refreshActnValid();
    });
    $('#down-move').click(function () {
        const sels = dpSheet.getSelections();
        const curRow = sels && sels[0] ? sels[0].row : dpSheet.getRowCount() - 1;
        SpreadJsObj.beginMassOperation(dpSheet);
        dpSheet.addRows(dpSheet.getRowCount(), 1);
        dpSheet.moveTo(curRow + 1, -1, dpSheet.getRowCount() - 1, -1, 1, -1, GC.Spread.Sheets.CopyToOptions.value);
        dpSheet.moveTo(curRow, -1, curRow + 1, -1, 1, -1, GC.Spread.Sheets.CopyToOptions.value);
        dpSheet.moveTo(dpSheet.getRowCount() - 1, -1, curRow, -1, 1, -1, GC.Spread.Sheets.CopyToOptions.value);
        dpSheet.deleteRows(dpSheet.getRowCount() - 1, 1);
        SpreadJsObj.endMassOperation(dpSheet);
        dpSheet.setSelection(curRow + 1, sels[0].col, sels[0].rowCount, sels[0].colCount);
        refreshActnValid();
    });

    $('#save').submit(function (e) {
        const data = [];
        for (let iRow = 0, iLen = dpSheet.getRowCount(); iRow < iLen; iRow++) {
            const payData = { order: iRow + 1 };
            try {
                for (const [iCol, col] of dpSheet.zh_setting.cols.entries()) {
                    switch (col.field) {
                        case 'name':
                            payData.name = dpSheet.getCell(iRow, iCol).text().replace(/[\r\n\t]/g, '');
                            break;
                        case 'ptype':
                            payData.ptype = dpSheet.getCell(iRow, iCol).value();
                            break;
                        case 'minus':
                            payData.minus = dpSheet.getCell(iRow, iCol).value();
                            if (payData.minus === null) payData.minus = false;
                            break;
                        case 'expr':
                        case 'sexpr':
                        case 'rexpr':
                            payData[col.field] = dpSheet.getCell(iRow, iCol).text().replace(/[\r\n\t=]/g, '');
                            break;
                    }
                }
            } catch (err) {
                e.preventDefault();
            }
            if (payData.name === '') {
                toastr.error('合同支付项必须含有名称，请检查第' + (iRow + 1) + '行');
                e.preventDefault();
            }
            data.push(payData);
        }
        $('input[name=data]').val(JSON.stringify(data));
    });
});