/**
 * 后台主页面js
 *
 * @date 2016/4/20
 * @author caiaolin
 * @version
 */

function autoFlashHeight(){
    $(".auto-height-1").height($(window).height()-115);
    $(".auto-height-2").height($(window).height()-154);
};
$(window).resize(autoFlashHeight);

$(document).ready(function() {

    // 菜单点击
    $("#menu > li > a").click(function() {
        var self = $(this);
        var subMenu = $(this).siblings('ul.sub-menu');
        if(subMenu.length > 0) {
            if(subMenu.is(":visible")) {
                self.find('.menu-arrow').removeClass('fa-angle-down').addClass('fa-angle-right');
                subMenu.slideUp('fast');
                self.parent().removeClass('active');
            }else{
                self.parent().addClass('active');
                self.find('.menu-arrow').removeClass('fa-angle-right').addClass('fa-angle-down');
                subMenu.slideDown('fast');
            }
        }
    });

    // 删除二次确认
    $(".table .delete").click(function() {
        var href = $(this).data('href');
        if (href == undefined || href == '') {
            return false;
        }
        var msg = $(this).data('msg');
        msg = msg == undefined ? '' : msg;
        var content = msg == '' ? '<h5>是否要删除记录' : '<h5>确定删除 <b>'+ msg +'</b>';
        content += ' ？</h5>';
        var dialog = $("#modal-dialog");

        dialog.find('.modal-body').html(content);
        dialog.find('.confirm').click(function() {
            window.location.href = href;
        });
        dialog.modal();
    });

    var datePicker = $(".date-picker");

    if(datePicker.length > 0) {
        datePicker.datetimepicker({
            language:  'zh-CN',
            weekStart: 1,
            todayBtn:  1,
            autoclose: 1,
            todayHighlight: 1,
            startView: 2,
            minView: 2,
            forceParse: 0
        });
    }

    var dateTimePicker = $(".datetime-picker");
    if (dateTimePicker.length > 0) {
        dateTimePicker.datetimepicker({
            language:  'zh-CN',
            weekStart: 1,
            todayBtn:  1,
            autoclose: 1,
            todayHighlight: 1,
            startView: 2,
            minView: 0,
            forceParse: 0
        });
    }

});

/**
 * 提示框
 *
 * @param string message
 * @param string type
 * @param string icon
 * @return void
 */
function toast(message, type, icon) {
    var toast = $(".toast");
    toast.addClass(type);
    toast.children('.message').html(message);
    var iconClass = 'fa-' + icon;
    toast.children('.icon').addClass(iconClass);
    toast.fadeIn(500);

    setTimeout(function() {
        toast.fadeOut('fast');
        toast.children('.message').text('');
        toast.children('.icon').removeClass(iconClass);
    }, 3000);
}

function trimInvalidChar(str) {
    return $.trim(str).replace(/\n/g, '').replace(/\r/g, '').replace(/\t/g, '');
}

//关闭等待窗口
function closeWaitingView() {
    var bgDiv = document.getElementById("bgDiv");
    var msgDiv = document.getElementById("msgDiv");
    //移除背景遮罩层div
    if(bgDiv != null){
        document.body.removeChild(bgDiv);
    }
    //移除中间信息提示层div
    if(msgDiv != null){
        document.body.removeChild(msgDiv);
    }
}
//显示等待窗口
function showWaitingView() {
    var msgw = 300; //提示窗口的宽度
    var msgh = 100; //提示窗口的高度

    var sWidth, sHeight;
    sWidth = document.body.clientWidth;
    sHeight = document.body.clientHeight;

    //背景遮罩层div
    var bgObj = document.createElement("div");
    bgObj.setAttribute('id', 'bgDiv');
    bgObj.style.zIndex = '9998';
    bgObj.style.position = "absolute";
    bgObj.style.top = "0px";
    bgObj.style.background = "#888";
    bgObj.style.filter = "progid:DXImageTransform.Microsoft.Alpha(style=3,opacity=25,finishOpacity=75";
    bgObj.style.opacity = "0.6";
    bgObj.style.left = "0px";
    bgObj.style.width = sWidth + "px";
    bgObj.style.height = sHeight + "px";
    document.body.appendChild(bgObj);

    //信息提示层div
    var msgObj = document.createElement("div");
    msgObj.style.zIndex = '9999';
    msgObj.setAttribute("id", "msgDiv");
    msgObj.setAttribute("align", "center");
    msgObj.style.position = "absolute";
    msgObj.style.font = "12px/1.6em Verdana, Geneva, Arial, Helvetica, sans-serif";
    msgObj.style.width = msgw + "px";
    msgObj.style.height = msgh + "px";
    msgObj.style.top = (document.documentElement.scrollTop + (sHeight - msgh) / 2) + "px";
    msgObj.style.left = (sWidth - msgw) / 2 + "px";
    document.body.appendChild(msgObj);

    //中间等待图标
    document.getElementById("msgDiv").innerHTML = '<i class="fa fa-spinner fa-pulse fa-3x fa-fw"></i>';
}

/**
 * 动态请求数据
 * @param {String} url - 请求链接
 * @param data - 提交数据
 * @param {function} successCallback - 返回成功回调
 * @param {function} errorCallBack - 返回失败回调
 */
const postData = function (url, data, successCallback, errorCallBack, showWaiting = false) {
    if (showWaiting) showWaitingView();
    $.ajax({
        type:"POST",
        url: url,
        data: {'data': JSON.stringify(data)},
        dataType: 'json',
        cache: false,
        timeout: 40000,
        beforeSend: function(xhr) {
            let csrfToken = Cookies.get('csrfToken');
            xhr.setRequestHeader('x-csrf-token', csrfToken);
        },
        success: function(result){
            if (result.err === 0) {
                if (successCallback) {
                    successCallback(result.data);
                }
            } else {
                toast('error: ' + result.msg, 'error', 'exclamation-circle');
                if (errorCallBack) {
                    errorCallBack(result.msg);
                }
            }
            if (showWaiting) closeWaitingView();
        },
        error: function(jqXHR, textStatus, errorThrown){
            toast('error ' + textStatus + " " + errorThrown, 'error', 'exclamation-circle');
            if (errorCallBack) {
                errorCallBack();
            }
            if (showWaiting) closeWaitingView();
        }
    });
};

/**
 * 动态请求数据(压缩数据)
 * @param {String} url - 请求链接
 * @param data - 提交数据
 * @param {function} successCallback - 返回成功回调
 * @param {function} errorCallBack - 返回失败回调
 */
const postDataCompress = function (url, data, successCallback, errorCallBack, showWaiting = false) {
    if (showWaiting) showWaitingView();
    $.ajax({
        type:"POST",
        url: url,
        data: {'data': LZString.compressToUTF16(JSON.stringify(data))},
        dataType: 'json',
        cache: false,
        timeout: 60000, // 导入清单Excel（10w行）预计需要时间
        beforeSend: function(xhr) {
            let csrfToken = Cookies.get('csrfToken');
            xhr.setRequestHeader('x-csrf-token', csrfToken);
        },
        success: function(result){
            if (result.err === 0) {
                if (successCallback) {
                    successCallback(result.data);
                }
            } else {
                toast('error: ' + result.msg, 'error', 'exclamation-circle');
                if (errorCallBack) {
                    errorCallBack(result.msg);
                }
            }
            if (showWaiting) closeWaitingView();
        },
        error: function(jqXHR, textStatus, errorThrown){
            toast('error ' + textStatus + " " + errorThrown, 'error', 'exclamation-circle');
            if (errorCallBack) {
                errorCallBack();
            }
            if (showWaiting) closeWaitingView();
        }
    });
};

/**
 * 动态请求数据
 * @param {String} url - 请求链接
 * @param data - 提交数据
 * @param {function} successCallback - 返回成功回调
 * @param {function} errorCallBack - 返回失败回调
 */
const postDataWithFile = function (url, formData, successCallback, errorCallBack, showWaiting = false) {
    if (showWaiting) showWaitingView();
    $.ajax({
        type:"POST",
        url: url,
        data: formData,
        dataType: 'json',
        cache: false,
        // 告诉jQuery不要去设置Content-Type请求头
        contentType: false,
        // 告诉jQuery不要去处理发送的数据
        processData: false,
        timeout: 5000,
        beforeSend: function(xhr) {
            let csrfToken = Cookies.get('csrfToken');
            xhr.setRequestHeader('x-csrf-token', csrfToken);
        },
        success: function(result){
            if (result.err === 0) {
                if (successCallback) {
                    successCallback(result.data);
                }
            } else {
                toast('error: ' + result.msg, 'error', 'exclamation-circle');
                if (errorCallBack) {
                    errorCallBack();
                }
            }
            if (showWaiting) closeWaitingView();
        },
        error: function(jqXHR, textStatus, errorThrown){
            toast('error ' + textStatus + " " + errorThrown, 'error', 'exclamation-circle');
            if (errorCallBack) {
                errorCallBack();
            }
            if (showWaiting) closeWaitingView();
        }
    });
};
