/**
 * 项目节相关js
 *
 * @author CaiAoLin
 * @date 2018/1/8
 * @version
 */
$(document).ready(function() {
    autoFlashHeight();
    const stdSpread = SpreadJsObj.createNewSpread($('.auto-height-2')[0]);
    const stdSheet = stdSpread.getActiveSheet();
    SpreadJsObj.initSheet(stdSheet, spreadSetting);
    const stdTree = createNewPathTree('base', treeSetting);
    postData(window.location.pathname + '/load', {dType: 'all'}, function (result) {
        stdTree.loadDatas(result);
        SpreadJsObj.loadSheetData(stdSheet, SpreadJsObj.DataType.Tree, stdTree);
        treeObj.refreshOperationValid(stdSheet);
    }, null, true);
    const treeObj = {
        refreshOperationValid: function (sheet, selection) {
            const setObjEnable = function (obj, enable) {
                if (enable) {
                    obj.removeClass('disabled').addClass('btn-outline-primary').removeClass('btn-outline-dark');
                } else {
                    obj.addClass('disabled').removeClass('btn-outline-primary').addClass('btn-outline-dark');
                }
            };

            const sel = selection ? selection[0] : sheet.getSelections()[0];
            const row = sel ? sel.row : -1;
            const tree = sheet.zh_tree;
            if (!tree) { return; }
            const node = sheet.zh_tree.nodes[row];
            const preNode = tree.getPreSiblingNode(node);
            const valid = !sheet.zh_setting.readOnly;

            setObjEnable($('#insert'), valid);
            setObjEnable($('#delete'), valid && node);
            setObjEnable($('#up-move'), valid && node && preNode);
            setObjEnable($('#down-move'), valid && node && !tree.isLastSibling(node));
            setObjEnable($('#up-level'), valid && node && tree.getParent(node));
            setObjEnable($('#down-level'), valid && node && preNode);
        },
        refreshTree: function (sheet, data) {
            SpreadJsObj.massOperationSheet(sheet, function () {
                const tree = sheet.zh_tree;
                // 处理删除
                if (data.delete) {
                    data.delete.sort(function (x, y) {
                        return y.deleteIndex - x.deleteIndex;
                    });
                    for (const d of data.delete) {
                        sheet.deleteRows(d.deleteIndex, 1);
                    }
                }
                // 处理新增
                if (data.create) {
                    const newNodes = data.create;
                    if (newNodes) {
                        newNodes.sort(function (a, b) {
                            return a.index - b.index;
                        });

                        for (const node of newNodes) {
                            sheet.addRows(node.index, 1);
                            SpreadJsObj.reLoadRowData(sheet, tree.nodes.indexOf(node), 1);
                        }
                    }
                }
                // 处理更新
                if (data.update) {
                    const rows = [];
                    for (const u of data.update) {
                        rows.push(tree.nodes.indexOf(u));
                    }
                    SpreadJsObj.reLoadRowsData(sheet, rows);
                }
                // 处理展开
                if (data.expand) {
                    const expanded = [];
                    for (const e of data.expand) {
                        if (expanded.indexOf(e) === -1) {
                            const posterity = tree.getPosterity(e);
                            for (const p of posterity) {
                                sheet.setRowVisible(tree.nodes.indexOf(p), p.visible);
                                expanded.push(p);
                            }
                        }
                    }
                }
            });
        },
        valueChanged: function (e, info) {
            const node = SpreadJsObj.getSelectObject(stdSheet);
            if (!node) {
                SpreadJsObj.reLoadRowData(stdSheet, info.row, 1);
                return;
            }
            const col = info.sheet.zh_setting.cols[info.col];
            if (info.newValue === info.oldValue) return;

            const updateData = stdTree.getNodeKeyData(node);
            updateData[col.field] = (!info.newValue || info.newValue === '')
                ? (col.field === 'node_type' ? 0 : null)
                : info.newValue;
            postData(window.location.pathname + '/update', {postType: 'update', updateData: updateData}, function (result) {
                const refreshData = stdTree.loadPostData(result);
                treeObj.refreshTree(stdSheet, refreshData);
                treeObj.refreshOperationValid(stdSheet)
            }, function() {
                SpreadJsObj.reLoadRowData(stdSheet, info.row, 1);
            });
        },
        add: function (sheet) {
            const tree = sheet.zh_tree;
            if (!tree) return;
            const node = SpreadJsObj.getSelectObject(sheet);

            const updateData = {postType: 'add'};
            if (node) updateData.updateData = tree.getNodeKey(node);
            postData(window.location.pathname + '/update', updateData, function (result) {
                const refreshNode = tree.loadPostData(result);
                treeObj.refreshTree(sheet, refreshNode);
                treeObj.refreshOperationValid(sheet);
            });
        },
        delete: function (sheet) {
            const tree = sheet.zh_tree;
            if (!tree) return;
            const sel = sheet.getSelections()[0];
            const node = sheet.zh_tree.nodes[sel.row];
            if (!node) return;
            const data = {postType: 'delete', updateData: tree.getNodeKey(node)};
            if (sel.rowCount > 1) {
                let count = 1;
                for (let r = 1; r < sel.rowCount; r++) {
                    const rNode = sheet.zh_tree.nodes[sel.row + r];
                    if (rNode.level > node.level) continue;
                    if ((rNode.level < node.level) || (rNode.level === node.level && rNode.pid !== node.pid)) {
                        toast('请选择同一清单下的节点，进行该操作');
                        return;
                    }
                    count += 1;
                }
                if (count > 1) {
                    data.count = count;
                }
            }

            postData(window.location.pathname + '/update', data, function (result) {
                const refreshNode = tree.loadPostData(result);
                treeObj.refreshTree(sheet, refreshNode);
                treeObj.refreshOperationValid(sheet);
            });
        },
        upMove: function (sheet) {
            const tree = sheet.zh_tree;
            if (!tree) return;
            const sel = sheet.getSelections()[0];
            const node = sheet.zh_tree.nodes[sel.row];
            if (!node) return;
            const data = {postType: 'up-move', updateData: tree.getNodeKey(node)};
            if (sel.rowCount > 1) {
                let count = 1;
                for (let r = 1; r < sel.rowCount; r++) {
                    const rNode = sheet.zh_tree.nodes[sel.row + r];
                    if (rNode.level > node.level) continue;
                    if ((rNode.level < node.level) || (rNode.level === node.level && rNode.pid !== node.pid)) {
                        toast('请选择同一清单下的节点，进行该操作');
                        return;
                    }
                    count += 1;
                }
                if (count > 1) {
                    data.count = count;
                }
            }

            postData(window.location.pathname + '/update', data, function (result) {
                const refreshNode = tree.loadPostData(result);
                treeObj.refreshTree(sheet, refreshNode);
                const sel = sheet.getSelections()[0];
                sheet.setSelection(tree.nodes.indexOf(node), sel.col, sel.rowCount, sel.colCount);
                treeObj.refreshOperationValid(sheet);
            });
        },
        downMove: function (sheet) {
            const tree = sheet.zh_tree;
            if (!tree) return;
            const sel = sheet.getSelections()[0];
            const node = sheet.zh_tree.nodes[sel.row];
            if (!node) return;
            const data = {postType: 'down-move', updateData: tree.getNodeKey(node)};
            if (sel.rowCount > 1) {
                let count = 1;
                for (let r = 1; r < sel.rowCount; r++) {
                    const rNode = sheet.zh_tree.nodes[sel.row + r];
                    if (rNode.level > node.level) continue;
                    if ((rNode.level < node.level) || (rNode.level === node.level && rNode.pid !== node.pid)) {
                        toast('请选择同一清单下的节点，进行该操作');
                        return;
                    }
                    count += 1;
                }
                if (count > 1) {
                    data.count = count;
                }
            }

            postData(window.location.pathname + '/update', data, function (result) {
                const refreshNode = tree.loadPostData(result);
                treeObj.refreshTree(sheet, refreshNode);
                const sel = sheet.getSelections()[0];
                sheet.setSelection(tree.nodes.indexOf(node), sel.col, sel.rowCount, sel.colCount);
                treeObj.refreshOperationValid(sheet);
            });
        },
        upLevel: function (sheet) {
            const tree = sheet.zh_tree;
            if (!tree) return;
            const sel = sheet.getSelections()[0];
            const node = sheet.zh_tree.nodes[sel.row];
            if (!node) return;
            const data = {postType: 'up-level', updateData: tree.getNodeKey(node)};
            if (sel.rowCount > 1) {
                let count = 1;
                for (let r = 1; r < sel.rowCount; r++) {
                    const rNode = sheet.zh_tree.nodes[sel.row + r];
                    if (rNode.level > node.level) continue;
                    if ((rNode.level < node.level) || (rNode.level === node.level && rNode.pid !== node.pid)) {
                        toast('请选择同一清单下的节点，进行该操作');
                        return;
                    }
                    count += 1;
                }
                if (count > 1) {
                    data.count = count;
                }
            }

            postData(window.location.pathname + '/update', data, function (result) {
                const refreshNode = tree.loadPostData(result);
                treeObj.refreshTree(sheet, refreshNode);
                treeObj.refreshOperationValid(sheet);
            });
        },
        downLevel: function (sheet) {
            const tree = sheet.zh_tree;
            if (!tree) return;
            const sel = sheet.getSelections()[0];
            const node = sheet.zh_tree.nodes[sel.row];
            if (!node) return;
            const data = {postType: 'down-level', updateData: tree.getNodeKey(node)};
            if (sel.rowCount > 1) {
                let count = 1;
                for (let r = 1; r < sel.rowCount; r++) {
                    const rNode = sheet.zh_tree.nodes[sel.row + r];
                    if (rNode.level > node.level) continue;
                    if ((rNode.level < node.level) || (rNode.level === node.level && rNode.pid !== node.pid)) {
                        toast('请选择同一清单下的节点，进行该操作');
                        return;
                    }
                    count += 1;
                }
                if (count > 1) {
                    data.count = count;
                }
            }

            postData(window.location.pathname + '/update', data, function (result) {
                const refreshNode = tree.loadPostData(result);
                treeObj.refreshTree(sheet, refreshNode);
                treeObj.refreshOperationValid(sheet);
            });
        },
        selectionChanged: function (e, info) {
            //SpreadJsObj.saveTopAndSelect(info.sheet, ckSpread);
            treeObj.refreshOperationValid(info.sheet, info.newSelections);
        }
    };
    stdSheet.bind(spreadNS.Events.ValueChanged, treeObj.valueChanged);
    stdSheet.bind(spreadNS.Events.SelectionChanged, treeObj.selectionChanged);
    $('#insert').bind('click', () => { treeObj.add(stdSheet); });
    $('#delete').bind('click', () => { treeObj.delete(stdSheet); });
    $('#up-move').bind('click', () => { treeObj.upMove(stdSheet); });
    $('#down-move').bind('click', () => { treeObj.downMove(stdSheet); });
    $('#up-level').bind('click', () => { treeObj.upLevel(stdSheet); });
    $('#down-level').bind('click', () => { treeObj.downLevel(stdSheet); });
    $('#export-excel').bind('click', () => {
        SpreadExcelObj.exportSpread2XlsxWithHeader(stdSpread, title + '.xlsx');
    });
});