'use strict';

module.exports = app => {
    // session验证中间件
    const sessionAuth = app.middlewares.sessionAuth();
    // 自动填充时间中间件
    const datetimeFill = app.middlewares.datetimeFill();
    // 权限验证中间件
    const permissionFilter = app.middlewares.permissionFilter();

    // 登录相关
    app.get('/', 'loginController.index');
    app.get('/logout', 'loginController.logout');
    app.post('/login', 'loginController.login');

    // 验证码相关
    app.get('/captcha', 'loginController.captcha');

    // 控制面板相关
    app.get('/dashboard', sessionAuth, 'dashboardController.index');


    // 管理员相关
    app.get('/manager', sessionAuth, permissionFilter, 'managerController.index');
    // app.get('/manager/add', sessionAuth, permissionFilter, 'managerController.add');
    app.get('/manager/profile', sessionAuth, permissionFilter, 'managerController.profile');
    app.get('/manager/delete/:id', sessionAuth, permissionFilter, 'managerController.delete');
    app.get('/manager/modify/:id', sessionAuth, permissionFilter, 'managerController.modify');
    app.post('/manager/save/:id', sessionAuth, permissionFilter, datetimeFill, 'managerController.save');
    app.post('/manager/profile', sessionAuth, permissionFilter, 'managerController.saveProfile');

    // 权限设置相关
    app.get('/permission', sessionAuth, permissionFilter, 'permissionController.index');
    app.get('/permission/add/:id', sessionAuth, permissionFilter, 'permissionController.add');
    app.get('/permission/modify/:id', sessionAuth, permissionFilter, 'permissionController.modify');
    app.get('/permission/delete/:id', sessionAuth, permissionFilter, 'permissionController.delete');
    app.post('/permission/save/:id', sessionAuth, permissionFilter, 'permissionController.save');

    // 维护设置
    app.get('/maintain', sessionAuth, permissionFilter, 'permissionController.maintain');
    app.post('/maintain/set/', sessionAuth, permissionFilter, 'permissionController.maintainSet');
    app.get('/maintain/stop/', sessionAuth, permissionFilter, 'permissionController.maintainStop');

    // 用户组相关
    app.get('/group', sessionAuth, permissionFilter, 'groupController.index');
    app.get('/group/add', sessionAuth, permissionFilter, 'groupController.add');
    app.get('/group/modify/:id', sessionAuth, permissionFilter, 'groupController.modify');
    app.post('/group/save/:id', sessionAuth, permissionFilter, datetimeFill, 'groupController.save');

    // sql执行相关
    app.get('/sql', sessionAuth, permissionFilter, 'sqlExecuteController.index');
    app.post('/sql/execute', sessionAuth, permissionFilter, 'sqlExecuteController.execute');

    // 白名单相关
    app.get('/white-list', sessionAuth, permissionFilter, 'whiteListController.index');
    app.get('/white-list/add', sessionAuth, permissionFilter, 'whiteListController.add');
    app.get('/white-list/modify/:id', sessionAuth, permissionFilter, 'whiteListController.modify');
    app.get('/white-list/delete/:id', sessionAuth, permissionFilter, 'whiteListController.delete');
    app.post('/white-list/save/:id', sessionAuth, permissionFilter, datetimeFill, 'whiteListController.save');

    // 通知相关
    app.get('/message', sessionAuth, permissionFilter, 'messageController.index');
    app.get('/message/add', sessionAuth, permissionFilter, 'messageController.add');
    app.get('/message/modify/:id', sessionAuth, permissionFilter, 'messageController.modify');
    app.get('/message/delete/:id', sessionAuth, permissionFilter, 'messageController.delete');
    app.post('/message/save/:id', sessionAuth, permissionFilter, datetimeFill, 'messageController.save');

    // 升级说明
    app.get('/version', sessionAuth, permissionFilter, 'versionController.index');
    app.get('/version/add/:id', sessionAuth, permissionFilter, 'versionController.add');
    app.get('/version/delete/:id', sessionAuth, permissionFilter, 'versionController.delete');
    app.post('/version/save/:id', sessionAuth, permissionFilter, datetimeFill, 'versionController.save');

    // 日志相关
    app.get('/log', sessionAuth, permissionFilter, 'logController.index');

    // ueditor相关
    app.get('/ueditor', sessionAuth, 'ueditorController.getConfig');

    // 项目相关
    app.get('/project', sessionAuth, permissionFilter, 'projectController.index');
    app.get('/project/add', sessionAuth, permissionFilter, 'projectController.add');
    app.get('/project/modify/:id', sessionAuth, permissionFilter, 'projectController.modify');
    app.get('/project/delete/:id', sessionAuth, permissionFilter, 'projectController.delete');
    app.get('/project/tender', sessionAuth, permissionFilter, 'projectController.tender');
    app.get('/project/setting', sessionAuth, permissionFilter, 'projectController.setting');
    app.post('/project/save-setting/:id', sessionAuth, permissionFilter, 'projectController.saveSetting');
    app.post('/project/save/:id', sessionAuth, permissionFilter, datetimeFill, 'projectController.save');
    app.post('/project/upload/qrcode', sessionAuth, permissionFilter, 'projectController.uploadQrCode');
    app.get('/project/office', sessionAuth, permissionFilter, 'projectController.office');
    app.post('/project/office/share', sessionAuth, permissionFilter, 'projectController.officeShare');
    app.get('/project/pageshow', sessionAuth, permissionFilter, 'projectController.pageshow');
    app.post('/project/pageshow/set', sessionAuth, permissionFilter, 'projectController.pageshowSet');
    app.get('/project/code/valid', sessionAuth, permissionFilter, 'projectController.codeValid');

    // 项目账户相关
    app.get('/project/account', sessionAuth, permissionFilter, 'projectAccountController.index');
    app.get('/project/account/add', sessionAuth, permissionFilter, 'projectAccountController.add');
    app.get('/project/account/modify/:id', sessionAuth, permissionFilter, 'projectAccountController.modify');
    app.get('/project/account/modify/:id/account', sessionAuth, permissionFilter, 'projectAccountController.modifyAccount');
    app.get('/project/account/modify/:id/password', sessionAuth, permissionFilter, 'projectAccountController.modifyPassword');
    app.get('/project/account/delete/:id', sessionAuth, permissionFilter, 'projectAccountController.delete');
    app.post('/project/account/exist', sessionAuth, permissionFilter, 'projectAccountController.accountExist');
    app.post('/project/account/save/:id', sessionAuth, permissionFilter, datetimeFill, 'projectAccountController.save');
    app.post('/project/account/reset/:id', sessionAuth, permissionFilter, 'projectAccountController.resetPassword');
    app.post('/project/account/switch', sessionAuth, permissionFilter, 'projectAccountController.accountSwitch');
    app.post('/project/account/unbind', sessionAuth, permissionFilter, 'projectAccountController.accountUnbind');
    app.get('/project/account/modify/:id/backdoor', sessionAuth, permissionFilter, 'projectAccountController.modifyBackdoor');

    // 项目 合同支付
    app.get('/project/dealpay', sessionAuth, permissionFilter, 'dealPayController.index');
    app.post('/project/dealpay/save', sessionAuth, permissionFilter, 'dealPayController.save');
    app.post('/project/dealpay/reset', sessionAuth, permissionFilter, 'dealPayController.reset');

    // 第三方接口设置
    app.get('/project/api2other', sessionAuth, permissionFilter, 'projectController.api2other');
    app.post('/project/api2other/save', sessionAuth, permissionFilter, 'projectController.api2otherSave');

    // 项目节相关
    app.get('/std-xmj', sessionAuth, permissionFilter, 'stdXmjController.index');
    app.get('/std-xmj/add/:id', sessionAuth, permissionFilter, 'stdXmjController.add');
    app.get('/std-xmj/detail/:id', sessionAuth, permissionFilter, 'stdXmjController.detail');
    app.get('/std-xmj/detail-upload/:id', sessionAuth, permissionFilter, 'stdXmjController.detailUpload');
    app.get('/std-xmj/delete/:id', sessionAuth, permissionFilter, 'stdXmjController.delete');
    app.get('/std-xmj/detail-export/:id', sessionAuth, permissionFilter, 'stdXmjController.detailExport');
    app.post('/std-xmj/get-children', sessionAuth, permissionFilter, 'stdXmjController.getChildren');
    app.post('/std-xmj/save/:id', sessionAuth, permissionFilter, datetimeFill, 'stdXmjController.save');
    app.post('/std-xmj/upload/:id', sessionAuth, permissionFilter, 'stdXmjController.uploadJSON');
    app.post('/std-xmj/detail/:id/load', sessionAuth, permissionFilter, 'stdXmjController.loadDetailData');
    app.post('/std-xmj/detail/:id/update', sessionAuth, permissionFilter, 'stdXmjController.updateDetailData');
    app.get('/std-xmj/detail/:id/download/:file', sessionAuth, permissionFilter, 'stdXmjController.exportDetailJsonData');

    // 工程量清单
    app.get('/std-gcl', sessionAuth, permissionFilter, 'stdGclController.index');
    app.get('/std-gcl/add/:id', sessionAuth, permissionFilter, 'stdGclController.add');
    app.get('/std-gcl/detail/:id', sessionAuth, permissionFilter, 'stdGclController.detail');
    app.get('/std-gcl/detail-upload/:id', sessionAuth, permissionFilter, 'stdGclController.detailUpload');
    app.get('/std-gcl/delete/:id', sessionAuth, permissionFilter, 'stdGclController.delete');
    app.get('/std-gcl/detail-export/:id', sessionAuth, permissionFilter, 'stdGclController.detailExport');
    app.post('/std-gcl/get-children', sessionAuth, permissionFilter, 'stdGclController.getChildren');
    app.post('/std-gcl/save/:id', sessionAuth, permissionFilter, datetimeFill, 'stdGclController.save');
    app.post('/std-gcl/upload/:id', sessionAuth, permissionFilter, 'stdGclController.uploadJSON');
    app.post('/std-gcl/detail/:id/load', sessionAuth, permissionFilter, 'stdGclController.loadDetailData');
    app.post('/std-gcl/detail/:id/update', sessionAuth, permissionFilter, 'stdGclController.updateDetailData');
    app.get('/std-gcl/detail/:id/download/:file', sessionAuth, permissionFilter, 'stdGclController.exportDetailJsonData');

    // 演示客户相关
    app.get('/customer/demo', sessionAuth, permissionFilter, 'customerController.demo');
    app.post('/customer/exist', sessionAuth, permissionFilter, 'customerController.customerExist');

    // 企业相关
    app.get('/enterprise', sessionAuth, permissionFilter, 'enterpriseController.index');
    app.get('/enterprise/add', sessionAuth, permissionFilter, 'enterpriseController.add');
    app.get('/enterprise/modify/:id', sessionAuth, permissionFilter, 'enterpriseController.modify');
    app.get('/enterprise/delete/:id', sessionAuth, permissionFilter, 'enterpriseController.delete');
    app.get('/enterprise/setting', sessionAuth, permissionFilter, 'enterpriseController.setting');
    app.post('/enterprise/save-setting/:id', sessionAuth, permissionFilter, 'enterpriseController.saveSetting');
    app.get('/enterprise/project', sessionAuth, permissionFilter, 'enterpriseController.project');
    app.post('/enterprise/save/:id', sessionAuth, permissionFilter, datetimeFill, 'enterpriseController.save');

    // 项目客户相关
    app.get('/enterprise/account', sessionAuth, permissionFilter, 'projectAccountController.enterpriseIndex');
    app.get('/enterprise/account/add', sessionAuth, permissionFilter, 'projectAccountController.enterpriseAdd');
    app.get('/enterprise/account/modify/:id', sessionAuth, permissionFilter, 'projectAccountController.enterpriseModify');
    app.get('/enterprise/account/delete/:id', sessionAuth, permissionFilter, 'projectAccountController.enterpriseDelete');
    app.post('/enterprise/account/exist', sessionAuth, permissionFilter, 'projectAccountController.enterpriseAccountExist');
    app.post('/enterprise/account/save/:id', sessionAuth, permissionFilter, datetimeFill, 'projectAccountController.enterpriseSave');
    app.post('/enterprise/account/reset/:id', sessionAuth, permissionFilter, 'projectAccountController.resetPassword');

    // 标准清单相关
    app.get('/bills-template', sessionAuth, permissionFilter, 'billsTemplateController.index');
    app.get('/bills-template/add/:id', sessionAuth, permissionFilter, 'billsTemplateController.add');
    app.get('/bills-template/detail/:id', sessionAuth, permissionFilter, 'billsTemplateController.detail');
    app.get('/bills-template/detail-upload/:id', sessionAuth, permissionFilter, 'billsTemplateController.detailUpload');
    app.get('/bills-template/delete/:id', sessionAuth, permissionFilter, 'billsTemplateController.delete');
    app.get('/bills-template/detail-export/:id', sessionAuth, permissionFilter, 'billsTemplateController.detailExport');
    app.post('/bills-template/get-children', sessionAuth, permissionFilter, 'billsTemplateController.getChildren');
    app.post('/bills-template/save/:id', sessionAuth, permissionFilter, datetimeFill, 'billsTemplateController.save');
    app.post('/bills-template/upload/:id', sessionAuth, permissionFilter, 'billsTemplateController.uploadJSON');
    app.post('/bills-template/detail/:id/load', sessionAuth, permissionFilter, 'billsTemplateController.loadDetailData');
    app.post('/bills-template/detail/:id/update', sessionAuth, permissionFilter, 'billsTemplateController.updateDetailData');
    app.get('/bills-template/detail/:id/download/:file', sessionAuth, permissionFilter, 'billsTemplateController.exportDetailJsonData');

    // 报表定制
    // app.get('/reportProject', sessionAuth, permissionFilter, 'reportProjectController.index');
    app.get('/report', sessionAuth, permissionFilter, 'reportProjectController.index');
    app.get('/report/modifyReport', sessionAuth, permissionFilter, 'reportController.index');
    app.post('/report/report_tpl_api/getOfficeResponsePeople', sessionAuth, permissionFilter, 'reportProjectController.getOfficeResponsePeople');
    app.post('/report/report_tpl_api/updateTreeRootNode', sessionAuth, permissionFilter, 'reportController.save');
    app.post('/report/report_tpl_api/partialUpdateTreeNode', sessionAuth, permissionFilter, 'reportController.partialUpdateTreeNode');
    app.post('/report/report_tpl_api/updateTreeDetailNode', sessionAuth, permissionFilter, 'reportController.updateDetailNode');
    app.post('/report/report_tpl_api/copyTreeNodes', sessionAuth, permissionFilter, 'reportController.copyNodes');
    app.post('/report/report_tpl_api/createNewRptGrp', sessionAuth, permissionFilter, 'reportController.createNewRptGrp');
    app.post('/report/report_tpl_api/updateRptGrp', sessionAuth, permissionFilter, 'reportController.updateRptGrp');
    app.post('/report/report_tpl_api/deleteRptGrp', sessionAuth, permissionFilter, 'reportController.deleteRptGrp');
    app.post('/report/report_tpl_api/getRptGrp', sessionAuth, permissionFilter, 'reportController.getRptGrp');
    app.post('/report/report_tpl_api/createDftRptTpl', sessionAuth, permissionFilter, 'reportController.createReportTemplate');
    app.post('/report/report_tpl_api/getRefRptTpl', sessionAuth, permissionFilter, 'reportController.getReportTemplate');
    app.post('/report/report_tpl_api/updateRptTpl', sessionAuth, permissionFilter, 'reportController.updateRptTpl');
    app.post('/report/report_tpl_api/getPreDefineCfg', sessionAuth, permissionFilter, 'reportController.getPreDefineCfg');
    app.post('/report/report_tpl_api/getPreviewPage', sessionAuth, permissionFilter, 'reportController.getPreviewPage');

    // 计价规范
    app.get('/valuation', sessionAuth, permissionFilter, 'valuationController.index');
    app.get('/valuation/add/:id', sessionAuth, permissionFilter, 'valuationController.add');
    app.get('/valuation/detail/:id', sessionAuth, permissionFilter, 'valuationController.detail');
    app.get('/valuation/delete/:id', sessionAuth, permissionFilter, 'valuationController.delete');
    app.post('/valuation/save/:id', sessionAuth, permissionFilter, datetimeFill, 'valuationController.save');
    app.post('/valuation/detail/:id/save', sessionAuth, permissionFilter, 'valuationController.detailSave');

    app.get('/help', sessionAuth, permissionFilter, 'helpController.index');
};
