'use strict';

/**
 * 清单列表数据模型
 *
 * @author CaiAoLin
 * @date 2018/1/10
 * @version
 */

module.exports = app => {

    class BillsTemplateList extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'bills_template_list';
        }

        /**
         * 规则
         *
         * @return {Object} - 返回规则
         */
        rule() {
            return {
                name: { type: 'string', required: true, allowEmpty: false, min: 2 },
                remark: { type: 'string', required: false, allowEmpty: true },
            };
        }

        /**
         * 查找数据
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();
        }

        /**
         * 保存数据
         *
         * @param {Object} data - post过来的数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 操作结果
         */
        async save(data, id = 0) {
            id = parseInt(id);
            id = isNaN(id) ? 0 : id;
            delete data._csrf;

            if (id > 0) {
                delete data.create_time;
                data.id = id;
            }

            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            return operate.affectedRows > 0;
        }

        /**
         * 删除清单列表
         *
         * @param {Number} id - 删除的id
         * @return {Boolean} - 返回删除的结果
         */
        async delete(id) {
            let result = false;

            try {
                id = parseInt(id);
                if (isNaN(id) || id <= 0) {
                    throw 'id格式错误';
                }
                // 初始化事务
                this.transaction = await this.db.beginTransaction();

                // 删除对应数据
                this.transaction.delete(this.tableName, { id });
                // 删除对应的清单数据
                this.ctx.service.billsTemplate.transaction = this.transaction;
                const codition = { list_id: id };
                this.ctx.service.billsTemplate.deleteByCondition(codition);

                result = await this.transaction.commit();
            } catch (error) {
                result = false;
                await this.transaction.rollback();
            }

            return result;
        }

    }

    return BillsTemplateList;
};
