'use strict';

/**
 * 用户组数据模型
 *
 * @author CaiAoLin
 * @date 2017/9/30
 * @version
 */

module.exports = app => {

    class Group extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'group';
        }

        /**
         * 校验规则
         *
         * @return {Object} - 返回校验规则
         */
        rule() {
            const rule = {
                name: { type: 'string', required: true, allowEmpty: false, message: '请填写名称' },
                remark: { type: 'string', allowEmpty: true },
                permission: { type: 'array', allowEmpty: false },
            };
            return rule;
        }

        /**
         * 获取用户组列表
         *
         * @return {Array} - 返回整理后的用户列表数据
         */
        async getGroupList() {
            const condition = {};
            const groupList = await this.getList(condition);

            if (groupList.length > 0) {
                for (const index in groupList) {
                    groupList[index].permission = await this.ctx.service.permission.getNameById(groupList[index].permission);
                }
            }
            return groupList;
        }

        /**
         * 保存数据
         *
         * @param {Object} data - post过来的数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 操作结果
         */
        async save(data, id = 0) {
            delete data._csrf;

            if (id > 0) {
                data.id = id;
                delete data.create_time;
            }
            data.permission = data.permission.join(',');
            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            const result = operate.affectedRows > 0;
            if (result) {
                const cacheKey = 'allGroup';
                this.cache.del(cacheKey);
            }
            return result;
        }

        /**
         * 获取所有数据
         *
         * @param {boolean} isCache - 是否使用缓存(默认使用)
         * @return {Array} - 返回数据
         */
        async getAllData(isCache = true) {
            const cacheKey = 'allGroup';
            const cacheData = await this.cache.get(cacheKey);
            if (cacheData !== null && isCache) {
                return JSON.parse(cacheData);
            }

            const dataList = await this.db.select(this.tableName);
            // 加入缓存
            if (dataList.length !== null) {
                this.cache.set(cacheKey, JSON.stringify(dataList), 'EX', app.config.cacheTime);
            }
            return dataList;
        }
    }

    return Group;
};
