'use strict';

/**
 * 日志数据模型
 *
 * @author CaiAoLin
 * @date 2017/10/30
 * @version
 */

module.exports = app => {

    class Log extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg 全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'log';
            // 日志类型
            this.type = {
                OPERATION: 1,
            };
        }

        /**
         * 校验规则
         *
         * @return {Object} - 返回校验规则
         */
        rule() {
            return {
                controller: { type: 'string', required: true, allowEmpty: false },
                action: { type: 'string', required: true, allowEmpty: false },
                operation: { type: 'string', required: true, allowEmpty: false },
                target_id: { type: 'int', required: true, allowEmpty: false },
            };
        }

        /**
         * 查找数据
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();

            data.type = parseInt(data.type);
            if (isNaN(data.type) || data.type <= 0) {
                return;
            }
            switch (data.type) {
                // 操作者
                case 1:
                    this.sqlBuilder.setAndWhere('operator', {
                        value: this.db.escape(data.keyword + '%'),
                        operate: 'like',
                    });
                    break;
                // 控制器
                case 2:
                    this.sqlBuilder.setAndWhere('controller', {
                        value: this.db.escape(data.keyword),
                        operate: '=',
                    });
                    break;
                // action
                case 3:
                    this.sqlBuilder.setAndWhere('action', {
                        value: this.db.escape(data.keyword),
                        operate: '=',
                    });
                    break;
                default:
                    break;
            }

        }

        /**
         * 记录日志
         *
         * @param {Object} data - 新增的日志数据
         * @return {boolean} - 返回记录结果
         */
        async addLog(data) {
            let result = false;
            try {
                // 校验数据
                const rule = this.rule();
                this.ctx.validate(rule, data);

                // 获取当前session用户数据
                const managerSession = this.ctx.session.managerSession;

                data.create_time = new Date().getTime() / 1000;
                data.operator = managerSession.username;
                data.type = this.type.OPERATION;

                const operate = await this.db.insert(this.tableName, data);

                result = operate.affectedRows > 0;
            } catch (error) {
                result = false;
            }

            return result;
        }
    }

    return Log;
};
