'use strict';

/**
 * 消息数据模型
 *
 * @author CaiAoLin
 * @date 2017/10/25
 * @version
 */

module.exports = app => {

    class Message extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'message';
            this.messageType = {
                1: '项目通知',
                2: '系统通知',
            };
            // 状态
            this.status = {
                SEND: 1,
                READY: 2,
            };
            // 状态显示
            this.displayStatus = [];
            this.displayStatus[this.status.READY] = '未发布';
            this.displayStatus[this.status.SEND] = '已发布';
        }

        /**
         * 校验规则
         *
         * @return {Object} - 返回校验规则
         */
        rule() {
            return {
                title: { type: 'string', required: true, allowEmpty: false },
                type: { type: 'string', required: true, allowEmpty: false },
                status: { type: 'string', required: true, allowEmpty: false },
                content: { type: 'string', required: true, allowEmpty: true },
            };
        }

        /**
         * 查找数据
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('type', {
                value: 2,
                operate: '=',
            });

            // // 通知类型筛选
            // if (data.message_type !== undefined && data.message_type !== '') {
            //     const messageType = data.message_type;
            //     if (isNaN(messageType)) {
            //         return;
            //     }
            //
            //     this.sqlBuilder.setAndWhere('type', {
            //         value: messageType,
            //         operate: '=',
            //     });
            // }
        }

        /**
         * 保存数据
         *
         * @param {Object} data - post过来的数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 操作结果
         */
        async save(data, id = 0) {
            id = parseInt(id);
            id = isNaN(id) ? 0 : id;
            delete data._csrf;
            const managerSessionData = this.ctx.session.managerSession;
            if (id > 0) {
                // 修改
                data.id = id;
                const messageData = await this.getDataById(id);
                if (parseInt(data.status) === this.status.SEND && messageData.status === this.status.READY) {
                    data.release_time = data.create_time;
                }
                delete data.create_time;
            } else {
                // 新增时设置创建者
                data.creator = managerSessionData.username;
                if (parseInt(data.status) === this.status.SEND) {
                    data.release_time = data.create_time;
                }
            }

            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            const result = operate.affectedRows > 0;

            return result;

            // id = parseInt(id);
            // id = isNaN(id) ? 0 : id;
            // delete data._csrf;
            // const managerSessionData = this.ctx.session.managerSession;
            //
            // // 处理发布时间
            // data.release_type = parseInt(data.release_type);
            // data.release_time = data.release_time === undefined ? 0 : Date.parse(new Date(data.release_time)) / 1000;
            // data.create_time = data.create_time === undefined ? 0 : Date.parse(new Date(data.create_time)) / 1000;
            // if (data.release_type === 2 && data.release_time === 0) {
            //     throw '请选择发布时间';
            // }
            //
            // data.status = data.release_time > data.create_time ? this.status.READY : this.status.SEND;
            // if (id > 0) {
            //     // 修改
            //     data.id = id;
            //     delete data.create_time;
            // } else {
            //     if (data.release_type === 2 && data.release_time <= data.create_time) {
            //         throw '发布时间不能小于当前时间';
            //     }
            //     // 新增时设置创建者
            //     data.creator = managerSessionData.username;
            // }
            // delete data.release_type;
            //
            // const operate = id === 0 ? await this.db.insert(this.tableName, data) :
            //     await this.db.update(this.tableName, data);
            //
            // const result = operate.affectedRows > 0;
            //
            // return result;
        }

        /**
         * 更新定时发布的数据
         *
         * @return {boolean} - 返回更新结果
         */
        async releaseSchedule() {
            const current = new Date().getTime() / 1000;
            const sql = 'UPDATE ?? SET `status` = ? WHERE `release_time` <= ? AND `status` = ?';
            const sqlParam = [this.tableName, this.status.SEND, current, this.status.READY];
            const operate = await this.db.query(sql, sqlParam);

            return operate.affectedRows > 0;
        }

    }

    return Message;
};
