'use strict';

/**
 * 项目数据模型
 *
 * @author CaiAoLin
 * @date 2017/11/8
 * @version
 */
//计量支付相关

const Jl = require('../lib/jl');
const cldOfficeConst = require('../const/cld_office');
module.exports = app => {

    class Project extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'project';
            // 状态相关
            this.status = {
                TRY: 1,
                NORMAL: 2,
                DISABLE: 3,
            };
            this.displayStatus = [];
            this.displayStatus[this.status.TRY] = '试用';
            this.displayStatus[this.status.NORMAL] = '正常';
            this.displayStatus[this.status.DISABLE] = '禁用';

            this.statusClass = [];
            this.statusClass[this.status.TRY] = 'warning';
            this.statusClass[this.status.NORMAL] = 'success';
            this.statusClass[this.status.DISABLE] = 'danger';
        }

        /**
         * 数据规则
         *
         * @param {String} name - 规则名称
         * @return {Object} - 返回数据规则
         */
        rule(name) {
            let rule = {};
            switch (name) {
                case 'project':
                    rule = {
                        code: { type: 'string', required: true, min: 2 },
                        name: { type: 'string', required: true, min: 2 },
                        remark: { type: 'string', required: false, allowEmpty: true },
                        manager_office: { type: 'string', required: true },
                        manager_id: { type: 'string', required: true },
                    };
                    break;
                case 'projectModify':
                    rule = {
                        name: { type: 'string', required: true, min: 2 },
                        remark: { type: 'string', required: false, allowEmpty: true },
                        status: { type: 'string', required: true, max: 1 },
                        // user_account: { type: 'string', required: true, min: 6 },
                        office: { type: 'string', required: true },
                        manager_office: { type: 'string', required: true },
                        manager_id: { type: 'string', required: true },
                        valuation: { type: 'array', required: true },
                        qrcode_json: { type: 'string', required: false, allowEmpty: true },
                    };
                    break;
                case 'projectSetting':
                    rule = {
                        standard_id: { type: 'string', required: true, min: 1 },
                        bill_id: { type: 'array', required: false },
                        chapter_id: { type: 'array', required: false },
                    };
                    break;
                default:
                    break;
            }
            return rule;
        }

        /**
         * 查询过虑
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data, managerSession) {
            this.initSqlBuilder();

            // 办事处筛选
            data.office = parseInt(data.office);
            data.office = isNaN(data.office) || data.office <= 0 ? 0 : data.office;
            if (data.office > 0) {
                this.sqlBuilder.setAndWhere('office', {
                    value: data.office,
                    operate: '=',
                });
            }

            // 状态筛选
            data.status = parseInt(data.status);
            data.status = isNaN(data.status) || data.status <= 0 ? 0 : data.status;
            if (data.status > 0) {
                this.sqlBuilder.setAndWhere('status', {
                    value: data.status,
                    operate: '=',
                });
            }
            if (managerSession.permission !== 'all' && managerSession && managerSession.office && managerSession.office !== cldOfficeConst.type.ZONGBU) {
                this.sqlBuilder.setNewOrWhere([{
                    field: 'manager_id',
                    value: managerSession.userID,
                    operate: '=',
                }, {
                    field: 'creator',
                    value: managerSession.userID,
                    operate: '=',
                }, {
                    field: 'office_share',
                    value: managerSession.office,
                    operate: 'find_in_set',
                }, {
                    field: 'office',
                    value: managerSession.office,
                    operate: '=',
                }]);
            }

            if (data.keyword === undefined || data.keyword === '') {
                return;
            }

            data.type = parseInt(data.type);
            // 过滤数据
            switch (data.type) {
                // 项目编号
                case 1:
                    this.sqlBuilder.setAndWhere('code', {
                        value: this.db.escape(data.keyword + '%'),
                        operate: 'like',
                    });
                    break;
                // 管理员
                case 2:
                    this.sqlBuilder.setAndWhere('user_account', {
                        value: this.db.escape(data.keyword + '%'),
                        operate: 'like',
                    });
                    break;
                default:
                    break;
            }
        }

        searchFilterByManager(filter, managerSession) {
            this.initSqlBuilder();

            // 状态筛选
            filter.status = parseInt(filter.status);
            filter.status = isNaN(filter.status) || filter.status <= 0 ? 0 : filter.status;
            if (filter.status > 0) {
                this.sqlBuilder.setAndWhere('status', {
                    value: filter.status,
                    operate: '=',
                });
            }

            // 过滤数据
            // console.log(filter.managerIds);
            if (filter.select_sale_responsor !== '') {
                filter.select_sale_responsor = parseInt(filter.select_sale_responsor);
                filter.select_sale_responsor = isNaN(filter.select_sale_responsor) || filter.select_sale_responsor <= 0 ? 0 : filter.select_sale_responsor;
                if (filter.select_sale_responsor > 0) {
                    this.sqlBuilder.setAndWhere('manager_id', {
                        value: filter.select_sale_responsor,
                        operate: '=',
                    });
                }
            } else if (filter.office) {
                filter.office = parseInt(filter.office);
                filter.office = isNaN(filter.office) || filter.office <= 0 ? 0 : filter.office;
                if (filter.office > 0) {
                    this.sqlBuilder.setAndWhere('office', {
                        value: filter.office,
                        operate: '=',
                    });
                }
            }

            if (filter.keyword !== undefined && filter.keyword !== null && filter.keyword !== '') {
                this.sqlBuilder.setAndWhere('name', {
                    value: this.db.escape(filter.keyword + '%'),
                    operate: 'like',
                });
            }

            if (managerSession.permission !== 'all' && managerSession && managerSession.office && managerSession.office !== cldOfficeConst.type.ZONGBU) {
                this.sqlBuilder.setNewOrWhere([{
                    field: 'manager_id',
                    value: managerSession.userID,
                    operate: '=',
                }, {
                    field: 'creator',
                    value: managerSession.userID,
                    operate: '=',
                }, {
                    field: 'office_share',
                    value: managerSession.office,
                    operate: 'find_in_set',
                }, {
                    field: 'office',
                    value: managerSession.office,
                    operate: '=',
                }]);
            }
        }

        /**
         * 保存设置数据
         *
         * @param {Object} data - 表单post数据
         * @param {Number} id - 修改id
         * @return {boolean} - 返回执行结果
         */
        async saveSetting(data, id = 0) {
            id = parseInt(id);
            delete data._csrf;

            data.standard_id = parseInt(data.standard_id);
            data.bill_id = data.bill_id !== undefined && data.bill_id !== null ? data.bill_id.join(',') : '';
            data.chapter_id = data.chapter_id !== undefined && data.chapter_id !== null ? data.chapter_id.join(',') : '';
            data.id = id;

            const operate = await this.db.update(this.tableName, data);

            return operate.affectedRows > 0;
        }

        async getNewCode() {
            let code = this.ctx.helper.generateProjectCodeString(5, 2);
            let count = await this.count({code: code});
            while (count > 0) {
                code = this.ctx.helper.generateProjectCodeString(5, 2);
                count = await this.count({code: code});
            }
            return code;
        }

        /**
         * 保存数据
         *
         * @param {Object} data - 表单post数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 返回执行结果
         */
        async save(data, id = 0) {
            let result = false;
            id = parseInt(id);
            delete data._csrf;

            //data.max_user = data.max_user !== '' ? parseInt(data.max_user) : 0;
            //data.valuation = data.valuation !== undefined && data.valuation !== null ? data.valuation.join(',') : '';
            // 初始化事务
            this.transaction = await this.db.beginTransaction();
            try {
                if (id > 0) {
                    data.id = id;
                    delete data.create_time;
                } else {
                    //TODO --项目编号--
                    //data.code = await this.getNewCode();

                    // 获取当前创建者数据
                    const sessionUserData = this.ctx.session.managerSession;
                    // const officeString = cldOfficeConst.list[sessionUserData.office] === undefined ?
                    //     '超级管理员' : cldOfficeConst.list[sessionUserData.office];
                    // data.creator = officeString + '-' + sessionUserData.username;
                    data.creator = sessionUserData.userID;

                    // data.office_share = data.office;

                    // 默认状态为试用
                    data.status = this.status.TRY;
                    //detele -测试中
                    // data.user_id=3;
                    // data.user_account='caipin'
                }
                //创建项目或者更新
                const operate = id === 0 ? await this.transaction.insert(this.tableName, data) :
                    await this.transaction.update(this.tableName, data);

                if (operate.affectedRows <= 0) {
                    throw '保存项目数据失败';
                }

                // 拉取计量支付管理用户---
                const jl = new Jl(this.ctx);
                result = await jl.getJlProjectAccountData(data.code);
                if(result===null){
                    throw '获取计量支付项目管理员失败';
                }
               
                // 先判断表中是否已存在该账号
                const projectId = id === 0 ? operate.insertId : id;
                const projectAccountData = await this.ctx.service.projectAccount.isAccountExist(result.account, projectId);
                if (projectAccountData) {
                    await this.ctx.service.projectAccount.checkAdminProjectAccount(projectAccountData);
                } else {
                    // 新增项目账号
                    this.ctx.service.projectAccount.transaction = this.transaction;//事务
                    const accountData=await this.ctx.service.projectAccount.saveAdminProjectAccount(result, projectId);

                    // 更新项目表
                    data.user_id=accountData.insertId;
                    data.user_account=result.account;
                    data.id=projectId;
                    await this.transaction.update(this.tableName, data);
                }


                
                    // 再判断账号属于customer还是sso
                    // const result = await this.ctx.service.customer.getDataInfo(data.user_account);
                    // if (result !== null && result.type === 2) {
                    //     // 添加到customer表
                    //     this.ctx.service.customer.transaction = this.transaction;
                    //     await this.ctx.service.customer.saveSSOUser(result.data);
                    // }
                result = await this.transaction.commit();

            } catch (error) {
                console.log(error);
                // 回滚
                await this.transaction.rollback();
            }

            return result;
        }

        /**
         * 假删除
         *
         * @param {Number} id - 删除的id
         * @return {Boolean} - 删除结果
         */
        async deleteProjectById(id) {
            const updateData = {
                status: this.status.DISABLE,
                id,
            };

            const result = await this.db.update(this.tableName, updateData);

            return result.affectedRows > 0;
        }

        /**
         * 获取项目列表
         *
         * @return {Array} - 返回项目数据
         */
        async getList() {
            // 获取当前项目信息
            const sessionEnterprise = this.ctx.session.sessionEnterpriseData;
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('enterprise_id', {
                value: sessionEnterprise.id,
                operate: '=',
            });
            this.sqlBuilder.columns = ['id', 'enterprise_id', 'name', 'status', 'create_time', 'user_id'];
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);

            const list = await this.db.query(sql, sqlParam);
            // for (const l of list) {
            //     l.category = l.category && l.category !== '' ? JSON.parse(l.category) : null;
            // }

            return list;
        }

        /**
         * 保存办事处共享
         *
         * @param {Object} data - 办事处列表
         * @param {Number} id - id
         * @return {Boolean} - 删除结果
         */
        async saveOfficeShare(data, id) {
            delete data._csrf;
            const updateData = {
                office_share: data.office_share.join(','),
                id,
            };

            const result = await this.db.update(this.tableName, updateData);

            return result.affectedRows > 0;
        }

        /**
         * 保存页面显示修改
         *
         * @param {Object} data - 办事处列表
         * @param {Number} id - id
         * @return {Boolean} - 删除结果
         */
        async savePageshow(data, id) {
            const ps = Object.keys(data).length > 0 ? JSON.stringify(data) : null;
            const updateData = {
                page_show: ps,
                id,
            };

            const result = await this.db.update(this.tableName, updateData);

            return result.affectedRows > 0;
        }

        /**
         * 获得一条项目数据
         * @param {*} field 字段
         * @param {*} value 值
         */
        async findOne(field,value) {
            let result = [];
            try{
                this.initSqlBuilder();
                this.sqlBuilder.setAndWhere(field, {
                    value: value,
                    operate: '=',
                });
                this.sqlBuilder.columns = ['id', 'code', 'name', 'status', 'create_time', 'user_id'];
                const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
                const projectData = await this.db.query(sql, sqlParam);
                //建设工程项目不存在，验证计量支付的项目编号
                if(projectData.length !==0){
                    result=projectData;
                }
            }catch (error) {
                console.log(error);
                result= [];
            }
            return result;
        }

        /**
         * 获取办事处id
         *
         * @param {Object} data - 办事处列表
         * @param {Number} id - id
         * @return {Boolean} - 删除结果
         */
        async getOfficeList(office, uid) {
            const sql = 'SELECT `office` FROM ?? WHERE FIND_IN_SET (?, `office_share`) OR `office` = ? OR `creator` = ? OR `manager_id` = ? GROUP BY `office`';
            const sqlParam = [this.tableName, office, office, uid, uid];
            return await this.db.query(sql, sqlParam);
        }
    }

    return Project;
};
