'use strict';

/**
 * Created by Tony on 2019/5/13.
 */

const BaseService = require('../base/base_service');
const JV = require('../reports/rpt_component/jpc_value_define');
const reportConst = require('../const/report');

module.exports = app => {

    class RptTpl extends BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'rpt_tpl';
            this.dataId = 'id';
        }

        async getTplById(id) {
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('id', {
                value: id,
                operate: '=',
            });
            this.sqlBuilder.columns = ['id', 'rpt_type', 'rpt_tpl_name', 'rpt_content'];
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
            const list = await this.db.query(sql, sqlParam);
            return list;
        }

        async copyAndCreateTemplate(refTpl) {
            let rst = null;
            this.transaction = await this.db.beginTransaction();
            try {
                const data = {
                    id: 0,
                    rpt_type: refTpl.rpt_type,
                    rpt_tpl_name: refTpl.rpt_tpl_name,
                    rpt_content: refTpl.rpt_content
                };
                rst = await this.transaction.insert(this.tableName, data);
                await this.transaction.commit();
            } catch (ex) {
                console.log(ex);
                // 回滚
                await this.transaction.rollback();
            }
            return rst;
        }

        _getRptCustomType (tplObj) {
            if (!tplObj || !tplObj[JV.NODE_CUSTOM_DEFINE]) return 0;
            if (tplObj[JV.NODE_CUSTOM_DEFINE][JV.NODE_CUS_AUDIT_SELECT].enable) {
                return reportConst.rptCustomType[JV.NODE_CUS_AUDIT_SELECT];
            } else if (tplObj[JV.NODE_CUSTOM_DEFINE][JV.NODE_CUS_GATHER_SELECT].enable) {
                return reportConst.rptCustomType[JV.NODE_CUS_GATHER_SELECT];
            } else if (tplObj[JV.NODE_CUSTOM_DEFINE][JV.NODE_CUS_COMPARE_SELECT].enable) {
                return reportConst.rptCustomType[JV.NODE_CUS_COMPARE_SELECT];
            } else if (tplObj[JV.NODE_CUSTOM_DEFINE][JV.NODE_CUS_STAGE_SELECT].enable) {
                return reportConst.rptCustomType[JV.NODE_CUS_STAGE_SELECT];
            }
            return 0;
        }
        _getRptDataType (tplObj) {
            if (!tplObj || !tplObj[JV.NODE_CUSTOM_DEFINE]) return 0;
            if (tplObj[JV.NODE_CUSTOM_DEFINE][JV.NODE_CUS_MATERIAL_SELECT].enable) {
                return reportConst.rptDataType[JV.NODE_CUS_MATERIAL_SELECT];
            }
            return 0;
        }

        async updateRptTpl(tplId, tplName, tplObj, cid) {
            let rst = null;
            this.transaction = await this.db.beginTransaction();
            try {
                console.log(this._getRptCustomType(tplObj));
                const data = {
                    id: tplId,
                    cid: cid,
                    rpt_c_type: this._getRptCustomType(tplObj),
                    rpt_d_type: this._getRptDataType(tplObj),
                    rpt_type: 0,
                    rpt_tpl_name: tplName, rpt_content: JSON.stringify(tplObj)
                };
                rst = await this.transaction.update(this.tableName, data);
                this.transaction.commit();
            } catch (ex) {
                console.log(ex);
                // 回滚
                await this.transaction.rollback();
            }
            return rst;
        }

    }
    return RptTpl;
};
