'use strict';

/**
 * sql执行业务逻辑
 *
 * @author CaiAoLin
 * @date 2017/10/13
 * @version
 */

module.exports = app => {

    class SqlExecute extends app.BaseService {

        /**
         * 校验规则
         *
         * @return {Object} - 返回校验规则
         */
        rule() {
            const rule = {
                sql: { type: 'string', required: true, allowEmpty: false, min: 12 },
            };

            return rule;
        }

        /**
         * 执行sql语句
         *
         * @param {String} sql - sql语句
         * @return {Array} - 返回数据为数组，第一个元素为字段数据，第二个元素为执行结果
         */
        async execute(sql) {
            // 过滤字符串
            sql = sql.trim();
            sql = sql.toLowerCase();

            const queryString = sql.slice(0, 6);
            if (queryString !== 'select') {
                throw '只能执行select相关代码';
            }

            // 防止慢查询
            if (sql.indexOf('limit') < 0) {
                sql += ' LIMIT 0,500';
            }
            const result = await this.db.query(sql);

            // 整理结果
            const filedList = [];
            if (result.length > 0) {
                for (const index in result[0]) {
                    filedList.push(index);
                }
            }

            return [filedList, result];
        }

    }

    return SqlExecute;
};
