'use strict';

/**
 * 版本数据模型
 *
 * @author CaiAoLin
 * @date 2017/10/25
 * @version
 */

module.exports = app => {

    class Version extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'version';
        }

        /**
         * 校验规则
         *
         * @return {Object} - 返回校验规则
         */
        rule() {
            return {
                name: { type: 'string', required: true, allowEmpty: false },
                content: { type: 'string', required: false, allowEmpty: true },
            };
        }

        /**
         * 查找数据
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();
        }

        /**
         * 保存数据
         *
         * @param {Object} data - post过来的数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 操作结果
         */
        async save(data, id = 0) {
            id = parseInt(id);
            id = isNaN(id) ? 0 : id;
            delete data._csrf;

            if (id > 0) {
                delete data.create_time;
                data.id = id;
            }

            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            return operate.affectedRows > 0;
        }
    }

    return Version;
};
